unit sfZeta;

{Common code for special functions: Zeta functions and polylogarithms}

interface

{$i std.inc}

{$ifdef BIT16}
{$N+}
{$endif}

{$ifdef NOBASM}
  {$undef BASM}
{$endif}


(*************************************************************************

 DESCRIPTION   :  Common code for Zeta functions and polylogarithms

 REQUIREMENTS  :  TP5.5-7, D2-D7/D9-D10/D12/D17-D18, FPC, VP, WDOSX

 EXTERNAL DATA :  ---

 MEMORY USAGE  :  ---

 DISPLAY MODE  :  ---

 REMARK        :  The unit can be compiled with TP5 but some functions generate
                  invalid operations due to TP5's brain-damaged usage of the FPU

 REFERENCES    :  References used in this unit, main index in sfBasic

                  [7] Cephes Mathematical Library, Version 2.8
                      http://www.moshier.net/#Cephes or http://www.netlib.org/cephes/
                 [10] R. Bulirsch, Numerical Calculation of Elliptic Integrals
                      and Elliptic Functions
                      Numerische Mathematik 7, 78-90, 1965
                 [14] SLATEC Common Mathematical Library, Version 4.1, July 1993
                      (general purpose mathematical and statistical routines written in Fortran 77)
                      http://www.netlib.org/slatec
                 [20] Special functions by Wayne Fullerton,
                      http://www.netlib.org/fn
                      Almost identical to the FNLIB subset of SLATEC [14]
                 [21] GNU Scientific Library, GSL-1.14 (March 2010),
                      http://www.gnu.org/software/gsl
                 [22] A.J. MacLeod, MISCFUN: A software package to compute uncommon special functions.
                      ACM Trans. on Math. Soft. 22 (1996), pp.288-301.
                      Fortran source: http://netlib.org/toms/757
                 [23] R.M. Corless, G.H. Gonnet, D.E.G. Hare, D.J. Jeffrey, D.E. Knuth,
                      On the Lambert W Function, Adv. Comput. Math., 5 (1996), pp. 329-359
                      http://www.apmaths.uwo.ca/~rcorless/frames/PAPERS/LambertW/LambertW.ps
                      or http://citeseerx.ist.psu.edu/viewdoc/summary?doi=10.1.1.33.3583
                 [24] D. Veberic, Having Fun with Lambert W(x) Function, 2010
                      http://arxiv.org/abs/1003.1628
                 [30] [NIST]: F.W.J. Olver, D.W. Lozier, R.F. Boisvert, C.W. Clark, NIST Handbook
                      of Mathematical Functions, Cambridge, 2010. Online resource: NIST Digital
                      Library of Mathematical Functions, http://dlmf.nist.gov/
                 [31] R.E. Crandall, Note on fast polylogarithm computation, 2006.
                      http://www.reed.edu/~crandall/papers/Polylog.pdf
                 [40] S.V. Aksenov et al., Application of the combined nonlinear-condensation
                      transformation to problems in statistical analysis and theoretical physics.
                      Computer Physics Communications, 150, 1-20, 2003.
                      Available from http://dx.doi.org/10.1016/S0010-4655(02)00627-6
                      or as e-print: http://arxiv.org/pdf/math/0207086v1
                 [41] C. Ferreira, J.L. Lopez, Asymptotic expansions of the Hurwitz-Lerch
                      zeta function, J. Math. Anal. Appl. 298 (2004), no. 1, 210-224.
                      http://dx.doi.org/10.1016/j.jmaa.2004.05.040 or from
                      http://pcmap.unizar.es/~chelo/investig/publicaciones/chelo/Hurwitz-Lerch/original.pdf
                 [47] M. Goano, Algorithm 745: Computation of the complete and incomplete
                      Fermi-Dirac integral. ACM TOMS, Vol.21, No.3, 1995, pp.221-232.
                      Fortran source available from http://netlib.org/toms/745
                 [50] A. Erdelyi et al., Higher Transcendental Functions Vol. I-III, California
                      Institute of Technology - Bateman Manuscript Project, 1953-1955,
                      Available via http://en.wikipedia.org/wiki/Bateman_Manuscript_Project

 Version  Date      Author      Modification
 -------  --------  -------     ------------------------------------------
 1.11.00  03.02.12  W.Ehrhardt  New unit with parts of sfgamma, sfmisc, sfbasic
 1.11.01  04.02.12  we          sfc_lerch
 1.11.02  05.02.12  we          sfc_lerch for z=-1: use lphi_aj instead of zetah
 1.11.03  06.02.12  we          sfc_dbeta
 1.11.04  06.02.12  we          sfc_dlambda
 1.11.05  07.02.12  we          sfc_lchi
 1.11.06  08.02.12  we          sfc_polylogr

 1.12.00  13.03.12  we          sfc_lchi(1,x) = arctanh(x)
 1.12.01  20.03.12  we          sfc_zetah with BoFHex

 1.13.00  09.06.12  we          sfc_polylog uses sfc_taylor
 1.13.01  09.06.12  we          sfc_fermi_dirac
 1.13.02  10.06.12  we          fix n>x in fd_asymp_exp_int
 1.13.03  11.06.12  we          sfc_zeta uses sfc_zetaint
 1.13.04  12.06.12  we          sfc_etaint
 1.13.05  13.06.12  we          sfc_fdm05, sfc_fdp05

 1.15.00  16.02.13  we          improved sfc_zetah

 1.16.00  20.03.13  we          fix sfc_zetah for j>NBoF
 1.16.01  27.03.13  we          sfc_zetah for s<0
 1.16.02  28.03.13  we          sfc_trilog
 1.16.03  29.03.13  we          TP5 fix for sfc_zeta
 1.16.04  29.03.13  we          sfc_zetah for a near 1 and s<0

 1.17.00  10.04.13  we          sfc_fdp15

 1.18.00  19.05.13  we          Prevent some wrong compiler optimizations for div by 3

 1.20.00  16.08.13  we          sfc_zeta for small s
 1.20.01  16.08.13  we          sfc_polylog with sfc_zetaint

 1.21.00  07.09.13  we          Improved sfc_zetah/hurwitz_formula
 1.21.01  11.09.13  we          Bernoulli polynomials sfc_bernpoly

***************************************************************************)


(*-------------------------------------------------------------------------
 (C) Copyright 2009-2013 Wolfgang Ehrhardt

 This software is provided 'as-is', without any express or implied warranty.
 In no event will the authors be held liable for any damages arising from
 the use of this software.

 Permission is granted to anyone to use this software for any purpose,
 including commercial applications, and to alter it and redistribute it
 freely, subject to the following restrictions:

 1. The origin of this software must not be misrepresented; you must not
    claim that you wrote the original software. If you use this software in
    a product, an acknowledgment in the product documentation would be
    appreciated but is not required.

 2. Altered source versions must be plainly marked as such, and must not be
    misrepresented as being the original software.

 3. This notice may not be removed or altered from any source distribution.
----------------------------------------------------------------------------*)

(*-------------------------------------------------------------------------
  This Pascal code uses material and ideas from open source and public
  domain libraries, see the file '3rdparty.ama' for the licenses.
---------------------------------------------------------------------------*)


function sfc_cl2(x: extended): extended;
  {-Return the Clausen function: integral(-ln(2*|sin(t/2)|),t=0..x) = Im(Li_2(exp(ix)))}

function sfc_ti2(x: extended): extended;
  {-Return the inverse tangent integral, ti2(x) = integral(arctan(t)/t, t=0..x)}

function sfc_dilog(x: extended): extended;
  {-Return dilog(x) = Re(Li_2(x)), Li_2(x) = -integral(ln(1-t)/t, t=0..x)}

function sfc_trilog(x: extended): extended;
  {-Return the trilogarithm function trilog(x) = Re(Li_3(x))}

function sfc_polylog(n: integer; x: extended): extended;
  {-Return the polylogarithm Li_n(x) of integer order; x<1 for n >= 0}

function sfc_polylogr(s, x: extended): extended;
  {-Return the polylogarithm Li_s(x) of real order; s>=0, |x|<=1, x<>1 if s<=1}

function sfc_pz(x: extended): extended;
  {-Return the prime zeta function P(x) = sum(1/p^x, p prime), x > 1}

function sfc_eta(s: extended): extended;
  {-Return the Dirichlet eta function}

function sfc_etaint(n: integer): extended;
  {-Return the Dirichlet function eta(n) for integer arguments}

function sfc_etam1(s: extended): extended;
  {-Return Dirichlet eta(s)-1}

function sfc_fermi_dirac(n: integer; x: extended): extended;
  {-Return the integer order Fermi-Dirac integral F_n(x) = 1/n!*integral(t^n/(exp(t-x)+1), t=0..INF)}

function sfc_fdm05(x: extended): extended;
  {-Return the complete Fermi-Dirac integral F(-1/2,x)}

function sfc_fdp05(x: extended): extended;
  {-Return the complete Fermi-Dirac integral F(1/2,x)}

function sfc_fdp15(x: extended): extended;
  {-Return the complete Fermi-Dirac integral F(3/2,x)}

function sfc_lerch(z,s,a: extended): extended;
  {-Return the Lerch transcendent Phi(z,s,a) = sum(z^n/(n+a)^s, n=0..INF), |z|<=1, s>=0, a>0; s>1 if z=1}

function sfc_zeta(s: extended): extended;
  {-Return the Riemann zeta function at s, s<>1}

function sfc_zeta1p(x: extended): extended;
  {-Return the Riemann zeta function at 1+x, x<>0}

function sfc_zetam1(s: extended): extended;
  {-Return Riemann zeta(s)-1, s<>1}

function sfc_zetah(s,a: extended): extended;
  {-Return the Hurwitz zeta function zetah(s,a) = sum(1/(i+a)^s, i=0..INF), s<>1, a>0}

function sfc_zetaint(n: integer): extended;
  {-Return zeta(n) for integer arguments, n<>1}

function sfc_dbeta(s: extended): extended;
  {-Return the Dirichlet beta function sum((-1)^n/(2n+1)^s, n=0..INF)}

function sfc_dlambda(s: extended): extended;
  {-Return the Dirichlet lambda function sum(1/(2n+1)^s, n=0..INF), s<>1}

function sfc_lchi(s, x: extended): extended;
  {-Return Legendre's Chi-function chi(s,x); s>=0, |x|<=1, x<>1 if s<=1}

function sfc_bernpoly(n: integer; x: extended): extended;
  {-Return the Bernoulli polynomial B_n(x), 0 <= n <= MaxBernoulli}

implementation

uses
  AMath,
  sfBasic, sfGamma;


(*
{---------------------------------------------------------------------------}
function cl2_mf(x: extended): extended;
  {-Return Clausen's integral, Cl_2(x) = integral(-ln(2*sin(t/2)),t=0..x) = Im(Li_2(exp(ix)))}
var
  t: extended;
  n: boolean;
const
  CA: array[0..15] of extended = (
        2.14269436376668844709e0,
        0.7233242812212579245e-1,
        0.101642475021151164e-2,
        0.3245250328531645e-4,
        0.133315187571472e-5,
        0.6213240591653e-7,
        0.313004135337e-8,
        0.16635723056e-9,
        0.919659293e-11,
        0.52400462e-12,
        0.3058040e-13,
        0.181969e-14,
        0.11004e-15,
        0.675e-17,
        0.42e-18,
        0.3e-19);
const
  xsmall = 7.314590396335798E-10; {Pi*sqrt(eps_x/2)}
begin
  {Ref: MISCFUN [22], function clausn}
  {NOTE: This is a Pascal reference implementation of the MISCFUN function.}
  {      The relative errors are large near the odd zeroes.}
  n := x<0.0;
  {Range reduction: cl2(x+2*n*Pi) = cl2(x)  and  cl2(Pi+x) = -cl2(Pi-x)}
  x := rem_2pi(abs(x));
  if x>Pi then begin
    x := (6.283172607421875-x)+0.1269975771147692529e-4;
    n := not n;
  end;
  if x<=0.0 then cl2_mf := 0.0
  else begin
    if x<xsmall then t := x*(1.0-ln(x))
    else begin
      t := 2.0*(sqr(x)/PiSqr-0.5);
      if t>1.0 then t:=1.0;
      t := x*(CSEvalX(t,CA, 16) - ln(x));
    end;
    if n then cl2_mf := -t else cl2_mf := t;
  end;
end;
*)


{---------------------------------------------------------------------------}
function sfc_cl2(x: extended): extended;
  {-Return the Clausen function: integral(-ln(2*|sin(t/2)|),t=0..x) = Im(Li_2(exp(ix)))}
var
  t,z: extended;
  n: integer;
const
  {h:= x-> 1+sum((-1)^(k-1)*bernoulli(2*k)/(2*k+1)!/(2*k)*x^(2*k),k=1..201);}
  {chebyshev(h(x),x=-Pi/2..Pi/2,0.1e-20); (Note: C1H[0] is doubled)}
  C1H: array[0..10] of THexExtW = (
         ($4B40,$522A,$CA82,$8236,$4000),  {2.0345941803613285109}
         ($92C1,$0D67,$7C45,$8E25,$3FF9),  {1.7351858820274076806E-2}
         ($50C0,$287F,$9C2D,$E75E,$3FF0),  {5.5162804260905214154E-5}
         ($DA31,$F30B,$7A72,$D593,$3FE9),  {3.9781646276597633104E-7}
         ($760C,$4541,$5693,$FD96,$3FE2),  {3.6901802891787812104E-9}
         ($B375,$9BE0,$8612,$AAA9,$3FDC),  {3.8804092136367920624E-11}
         ($280D,$9A61,$20B1,$F817,$3FD5),  {4.4069697689724379977E-13}
         ($26AB,$BC64,$5496,$BE1D,$3FCF),  {5.2767393780251196689E-15}
         ($05E0,$4305,$02DA,$9775,$3FC9),  {6.5684035804606950287E-17}
         ($EC0B,$5F67,$98E4,$F8A0,$3FC2),  {8.4238217037971627742E-19}
         ($7F28,$86A9,$64C0,$D0F4,$3FBC)); {1.1061967718533347427E-20}
const
  {g:=x->-I*(polylog(2,exp(I*(x+Pi)))-polylog(2,exp(-I*(x+Pi))))/2;}
  {chebyshev(g(x)/x,x=-Pi/2..Pi/2, 0.1e-20);(Note: C2H[0] is doubled)}
  C2H: array[0..15] of THexExtW = (
         ($CF66,$9C5A,$DDE7,$A39B,$BFFF),  {-1.2781941777145306826}
         ($1E33,$1215,$4E31,$E133,$3FFA),  { 5.4980569301851715639E-2}
         ($B1E0,$A525,$327B,$FBFD,$3FF4),  { 9.6126194595060642936E-4}
         ($AA4E,$B610,$7538,$8672,$3FF0),  { 3.2054686822550476557E-5}
         ($3403,$0F98,$F4BE,$B26F,$3FEB),  { 1.3294616954255450141E-6}
         ($2652,$4EBE,$5190,$8558,$3FE7),  { 6.2093601824397519462E-8}
         ($B572,$95C3,$8459,$D710,$3FE2),  { 3.1296006563911126723E-9}
         ($45E0,$43FC,$E98E,$B6E7,$3FDE),  { 1.6635195381926697759E-10}
         ($0F2D,$F984,$7A08,$A1C9,$3FDA),  { 9.1965272507194254495E-12}
         ($C5F5,$A603,$7382,$937E,$3FD6),  { 5.2400377387584500936E-13}
         ($47E7,$1D77,$C855,$89B8,$3FD2),  { 3.0580384187365945412E-14}
         ($640A,$EC19,$63AE,$831F,$3FCE),  { 1.8196918249487950988E-15}
         ($6A2A,$2898,$0C67,$FDBC,$3FC9),  { 1.1003982631962615223E-16}
         ($A44C,$F379,$66A5,$F8DA,$3FC5),  { 6.7451775715424687730E-18}
         ($5366,$B641,$745C,$F6E8,$3FC1),  { 4.1827846515724770346E-19}
         ($4D9A,$9F08,$9D0C,$F770,$3FBD)); { 2.6198718087610612748E-20}
        {($FA33,$3EBE,$22C1,$FA25,$3FB9)}  { 1.6553211620337321920E-21}
var
  C1: array[0..10] of extended absolute C1H;
  C2: array[0..15] of extended absolute C2H;
begin
  {See MISCFUN [22], function clausn for formulas and hints. As pointed}
  {out, only absolute accuracy can be guaranteed close to the zeroes.  }
  {Observed relative errors are e.g. 1700 eps_x for abs(x-Pi) ~ 6.7E-4,}
  {but even for abs(x-Pi) ~ 0.03 they are still about 128 eps_x.}

  {Therefore two separate Chebyshev expansions are calculated, one for }
  {z in (-Pi/2, Pi/2) the other for z in (Pi/2, 3*Pi/2), where z is the}
  {reduced argument. For z=0 or z=Pi cl2 is zero. Calculations are done}
  {using Maple with Digits:=50; and t_rcalc to convert to Hex/Extended.}
  {Note that both approximations are done for even functions, and there}
  {are only the even Chebyshev polynomials, so the argument for CSEvalX}
  {is 2(x/(Pi/2))^2 - 1 with the calculated coefficients.}

  {Argument reduction x mod Pi, |z| <= Pi/2}
  n := rem_pio2(0.5*x,z);
  z := 2.0*z;
  t := abs(z);

  if t=0.0 then begin
    {if x is an exact multiple of Pi then cl2(x)=0}
    sfc_cl2 := 0.0
  end
  else begin
    if odd(n) then begin
      {Use approximation for (Pi/2, 3*Pi/2) centered at Pi}
      if t<=1e-10 then begin
        {cl2(x+Pi) = x*(-ln(2) + 1/24*x^2 + 1/960*x^4+ O(x^6))}
        sfc_cl2 := -ln2*z;
      end
      else begin
        {Use Chebyshev expansion for cl2(x+Pi)/x calculated from}
        {cl2(x) = Im(Li_2(exp(ix) = -i*(Li_2(exp(ix))-Li_2(exp(-ix)))/2}
        sfc_cl2 := CSEvalX(2.0*(sqr(2.0*z/Pi)-0.5),C2,16) * z;
      end;
    end
    else begin
      {Use approximation for (-Pi/2, Pi/2)}
      if t<=1e-9 then begin
        {cl2(x) = x*(1-ln(x) + 1/72*x^3 + 1/14400*x^5 + O(x^6) }
        sfc_cl2 := z*(1.0-ln(t));
      end
      else begin
        {Use Chebyshev expansion calculated from [22], (3) or HMF[1], 27.8.2}
        {cl2(x) = -x*ln|x| + x - sum((-1)^k*B_2k/(2k+1)!/(2k)*x^(2k+1),k=1..Inf)}
        sfc_cl2 := (CSEvalX(2.0*(sqr(2.0*z/Pi)-0.5),C1,11)-ln(t))*z;
      end;
    end;
  end;
end;


{---------------------------------------------------------------------------}
function sfc_dilog(x: extended): extended;
  {-Return dilog(x) = Re(Li_2(x)), Li_2(x) = -integral(ln(1-t)/t, t=0..x)}
var
  t: extended;
const
  xbig = 1.8446744073709551616e19;
const
  nsp = 24;
  csp: array[0..nsp-1] of extended = (
         0.1527365598892405872946684910028e+00,
         0.8169658058051014403501838185271e-01,
         0.5814157140778730872977350641182e-02,
         0.5371619814541527542247889005319e-03,
         0.5724704675185826233210603054782e-04,
         0.6674546121649336343607835438589e-05,
         0.8276467339715676981584391689011e-06,
         0.1073315673030678951270005873354e-06,
         0.1440077294303239402334590331513e-07,
         0.1984442029965906367898877139608e-08,
         0.2794005822163638720201994821615e-09,
         0.4003991310883311823072580445908e-10,
         0.5823462892044638471368135835757e-11,
         0.8576708692638689278097914771224e-12,
         0.1276862586280193045989483033433e-12,
         0.1918826209042517081162380416062e-13,
         0.2907319206977138177795799719673e-14,
         0.4437112685276780462557473641745e-15,
         0.6815727787414599527867359135607e-16,
         0.1053017386015574429547019416644e-16,
         0.1635389806752377100051821734570e-17,
         0.2551852874940463932310901642581e-18,
         0.3999020621999360112770470379519e-19,
         0.6291501645216811876514149171199e-20);
{$ifdef VER5X}
  const Pi26H: THexExtW = ($9918,$983E,$3312,$D28D,$3FFF);
  var   Pi26 : extended absolute Pi26H;
{$else}
  const Pi26 = Pi*Pi/6.0;
{$endif}
begin
  {Ref: W. Fullerton [14] and [20], file dspenc.f}

  {Note that there is some confusion about the naming: some authors}
  {and/or computer algebra systems use dilog(x)= Li_2(1-x) and then}
  {call Li_2(x) Spence function/integral or similar.}

  {The imaginary part Im(Li_2(x) is 0 for x<=1 and -Pi*ln(x) for x>1}

  if x>2.0 then begin
    t := 2.0*Pi26 - 0.5*sqr(ln(x));
    if x>=xbig then sfc_dilog := t
    else sfc_dilog := t - (1.0 + CSEvalX(4.0/x-1.0, csp, nsp))/x;
  end
  else if x>1.0 then begin
    {1 < x <= 2}
    sfc_dilog := Pi26 - 0.50*ln(x)*ln(sqr((x-1.0))/x)
                 + (x-1.0)*(1.0 + CSEvalX(4.0*(x-1.0)/x-1.0, csp, nsp))/x;
  end
  else if x>0.5 then begin
    {0.5 < x <= 1}
    if x=1.0 then sfc_dilog := Pi26
    else sfc_dilog := Pi26 - ln(x)*ln1p(-x) - (1.0-x)*(1.0+CSEvalX(4.0*(1.0-x)-1.0, csp, nsp));
  end
  else if x>=0.0 then begin
    {0 <= x <= 0.5}
    sfc_dilog := x*(1.0 + CSEvalX(4.0*x-1.0, csp, nsp));
  end
  else if x>-1.0 then begin
    {-1 < x < 0}
    sfc_dilog := -0.5*sqr(ln(1.0-x)) - x*(1.0+CSEvalX(4.0*x/(x-1.0)-1.0, csp, nsp))/(x-1.0);
  end
  else begin
    {x <= -1.0}
    t := ln1p(-x);
    t := -Pi26 - 0.5*t*(2.0*ln(-x)-t);
    if x <= -xbig then sfc_dilog := t
    else begin
      sfc_dilog := t + (1.0 + CSEvalx(4.0/(1.0-x)-1.0, csp, nsp))/(1.0-x);
    end;
  end;
end;


{---------------------------------------------------------------------------}
function sfc_trilog(x: extended): extended;
  {-Return the trilogarithm function trilog(x) = Re(Li_3(x))}
var
  a,b: extended;
begin
  if IsNan(x) then begin
    {$ifopt R+}
      if RTE_ArgumentRange>0 then RunError(byte(RTE_ArgumentRange));
    {$endif}
    sfc_trilog := Nan_x;
    exit;
  end;
  if abs(x)>1.0 then begin
    b := 1.0/x;
    if abs(b) > sqrt_epsh then a := sfc_polylog(3,b)
    else begin
      {large |x|, use polylog(3,b) = b*(1 + b/8 + b^2/27 + O(b^3))}
      a := b*(1.0 + 0.125*b);
    end;
    {transform to intervall (-1,1) via inversion formulas}
    if x>0.0 then begin
      {This is the 'new' part for x>1. The real part of the RHS of}
      {Crandall's [31] formula (1.3) is Pi^2/3 * ln(x) - ln^3(x)/6}
      b := ln(x);
      b := (PiSqr - 0.5*sqr(b))*b/THREE;
      sfc_trilog := a + b;
    end
    else begin
      {Could be handled by sfc_polylog, but this branch}
      {avoids one recursive call with sfc_polylog(1/x):}
      {Re(RHS) becomes -Pi^2/6 * ln(-x) - ln^3(-x)/6 }
      b := ln(-x);
      b := (PiSqr + sqr(b))*b/6.0;
      sfc_trilog := a - b;
    end;
  end
  else begin
    {-1 <= x <= 1}
    sfc_trilog := sfc_polylog(3,x);
  end;
end;


{---------------------------------------------------------------------------}
function polylogseries(n: integer; x: extended): extended;
  {-polylog via series, |n|>2, |x| < 1}
var
  e,k,p,s,t,w,y,z: extended;
begin
  {Standard series sum(x^k/k^n, k=1..) calculated with sum2 algorithm.}
  {Actually sum2 is needed for x < 0, especially for large negative n.}
  p := x;
  s := p;
  e := 0.0;
  k := 1.0;
  repeat
    p := p*x;
    k := k+1.0;
    if n>0 then t := p/power(k,n)
    else t := p*power(k, -n);
    {(w,y) = TwoSum(t,s)}
    w := t+s;
    z := w-t;
    y := (t-(w-z)) + (s-z);
    {sum of errors}
    e := e+y;
    s := w;
  until abs(t) < eps_x*abs(s);
  polylogseries := s+e;
end;


{---------------------------------------------------------------------------}
function cr15_complex(n: integer; x: extended): extended;
  {-Compute polylog(n,x) using Crandall[31] (1.5); n<-1, |log(x)| < 2*Pi}
type
  TComplex = record r,i: extended; end;
var
  lx,s,p,t: TComplex;
  bk: extended;
  k: integer;
  done: boolean;
begin
  {This function uses Crandall[31] (1.5) with (inline) complex arithmetic}
  done := false;
  k    := 0;
  {lx = ln(x)}
  lx.r := ln(abs(x));
  lx.i := Pi;
  p.r  := 1.0;
  p.i  := 0.0;
  s.r  := sfc_bernoulli(1-n)/(1-n);
  s.i  := 0.0;
  repeat
    inc(k);
    t.r := p.r*lx.r - p.i*lx.i;
    t.i := p.r*lx.i + p.i*lx.r;
    p.r := t.r/k;
    p.i := t.i/k;
    bk := sfc_bernoulli(k-n+1)/(k-n+1);
    if bk<>0.0 then begin
      t.r := p.r*bk;
      t.i := p.i*bk;
      s.r := s.r + t.r;
      s.i := s.i + t.i;
      done := (abs(t.r)+abs(t.i)) < eps_x*(abs(s.r)+abs(s.i));
    end;
  until done;
  {rt.r = Re(power(-lx, n-1))}
  p.r := hypot(lx.r,lx.i);
  p.i := arctan2(-lx.i,-lx.r);
  t.r := power(p.r,n-1)*cos((n-1)*p.i);
  cr15_complex := sfc_fac(-n)*t.r - s.r;
end;


{---------------------------------------------------------------------------}
function polylogneg(n: integer; x: extended): extended;
  {-Polylog for n<0 and x<1}
var
  s,p,lx,t: extended;
  k: integer;
  done: boolean;
const
  a4: array[0..3] of extended = (1, 11, 11, 1);
  a5: array[0..4] of extended = (1, 26, 66, 26, 1);
  a6: array[0..5] of extended = (1, 57, 302, 302, 57, 1);
  a7: array[0..6] of extended = (1, 120, 1191, 2416, 1191, 120, 1);
  a8: array[0..7] of extended = (1, 247, 4293, 15619, 15619, 4293, 247, 1);
  a9: array[0..8] of extended = (1, 502, 14608, 88234, 156190, 88234, 14608, 502, 1);
begin
  if x = 1.0 then polylogneg := PosInf_x
  else if n >= -9 then begin
    p := power(1.0-x,1-n);
    case n of
       -9: s := PolEvalX(x, a9, 9);
       -8: s := PolEvalX(x, a8, 8);
       -7: s := PolEvalX(x, a7, 7);
       -6: s := PolEvalX(x, a6, 6);
       -5: s := PolEvalX(x, a5, 5);
       -4: s := PolEvalX(x, a4, 4);
       -3: s := 1.0 + x*(4.0 + x);
       -2: s := 1.0 + x;
      else s := 1.0
    end;
    polylogneg := x/p*s;
  end
  else if x=-1.0 then polylogneg := -sfc_etaint(n)
  else if abs(x) <= 0.25 then polylogneg := polylogseries(n,x)
  else if (x > 4.0) or (x < -11.5)  then begin
    s := polylogneg(n,1.0/x);
    if odd(n) then polylogneg := s
    else polylogneg := -s;
  end
  else if x>0 then begin
    {Use Crandall[31] (1.5) with real arithmetic}
    lx := ln(x);
    k  := 0;
    p  := 1.0;
    s  := sfc_bernoulli(1-n)/(1-n);
    done := false;
    repeat
      inc(k);
      p := p*lx/k;
      t := sfc_bernoulli(k-n+1);
      if t<>0.0 then begin
        t := t/(k-n+1)*p;
        s := s + t;
        done := abs(t)<abs(s)*eps_x;
      end;
    until done;
    polylogneg := sfc_fac(-n)*power(-lx, n-1) - s;
  end
  else begin
    {Same formula as above but with complex arithmetic}
    polylogneg := cr15_complex(n,x);
  end;
end;


{---------------------------------------------------------------------------}
function sfc_polylog(n: integer; x: extended): extended;
  {-Return the polylogarithm Li_n(x) of integer order; x<1 for n >= 0}
var
  h,p,s,t,z: extended;
  j,k: integer;
begin

  {Ref: Cephes [7] function polylog in file polylog.c and}
  {Crandall [31], Note on fast polylogarithm computation.}

  if IsNan(x) or ((n>0) and (x>1.0)) then begin
    {$ifopt R+}
      if RTE_ArgumentRange>0 then RunError(byte(RTE_ArgumentRange));
    {$endif}
    sfc_polylog := Nan_x;
    exit;
  end;

  if x=0.0 then begin
    sfc_polylog := 0.0;
    exit;
  end
  else if n<0 then begin
    if x=1.0 then begin
      {$ifopt R+}
        if RTE_ArgumentRange>0 then RunError(byte(RTE_ArgumentRange));
      {$endif}
      sfc_polylog := PosInf_x;
    end
    else sfc_polylog := polylogneg(n,x);
    exit;
  end
  else if n<=2 then begin
    if (n<2) and (x=1.0) then begin
      {$ifopt R+}
        if RTE_ArgumentRange>0 then RunError(byte(RTE_ArgumentRange));
      {$endif}
      sfc_polylog := Nan_x;
      exit;
    end;
    case n of
        0: sfc_polylog := x/(1.0-x);
        1: sfc_polylog := -ln1p(-x);
      else sfc_polylog := sfc_dilog(x);
    end;
    exit;
  end
  else if (n>40) and (x > -1.0) then begin
    if n > 63 then sfc_polylog := x
    else sfc_polylog := x*(1.0 + x*ldexp(1,-n));
    exit;
  end;

  {here x<=1 and n>2}
  if x=1.0 then sfc_polylog := sfc_zetaint(n)
  else if x=-1.0 then sfc_polylog := -sfc_etaint(n)
  else if x < -1.0 then begin
    {Inversion formula for x < -1, see Cephes[7] and use Li_k(-1) = -eta(k)}
    {Li_n(-z) + (-1)^n*Li_n(-1/z) = -ln(z)^n/n! - 2*sum(ln(z)^(n-2k)/(n-2k)!*eta(2k), k=1..n div 2)}
    {See also: L. Lewin, Polylogarithms and associated functions, 1981, (7.20)}
    {or http://functions.wolfram.com/10.08.17.0060.01}
    z := ln(-x);
    s := 0.0;
    for k:=1 to n div 2 do begin
      j := 2*k;
      p := -sfc_etaint(j);
      j := n-j;
      if j>0 then p := sfc_taylor(z,j)*p;
      s := s + p;
    end;
    t := sfc_polylog(n, 1.0/x);
    if odd(n) then t := -t;
    sfc_polylog := 2.0*s - t - sfc_taylor(z,n);
    exit;
  end
  else if (x < 0.5) and (x > -0.875) then begin
    {Use standard series sum(x^k/k^n, k=1..)}
    sfc_polylog := polylogseries(n,x);
  end
  else if x<0 then begin
    {For -1 < x <= -0.875 use recursive calls: Crandall[31], formula (1.2)}
    s := sfc_polylog(n,x*x);
    t := sfc_polylog(n,-x);
    sfc_polylog := exp2(1-n)*s - t;
  end
  else begin
    {Here x>=0.5, use Crandall[31], formula (1.4)}
    h := 0.0;
    for k:=pred(n) downto 1 do h := h + 1.0/k;
    z := ln(x);
    {Note that -ln(-ln(x)) > 0.3665}
    h := h - ln(-z);
    p := 1.0;
    s := sfc_zetaint(n);
    for j:=1 to n+1 do begin
      p := p*z/j;
      if j=n-1 then s := s + h*p
      else s := s + sfc_zetaint(n-j)*p;
    end;
    j := n+3;
    z := z*z;
    repeat
      p := p*z/(longint(j)*pred(j));
      h := sfc_zeta(n-j)*p;
      s := s + h;
      if abs(h) < eps_x*abs(s) then begin
        sfc_polylog := s;
        exit;
      end;
      inc(j,2);
    until false;
  end;
end;


{---------------------------------------------------------------------------}
function sfc_polylogr(s, x: extended): extended;
  {-Return the polylogarithm Li_s(x) of real order; s>=0, |x|<=1, x<>1 if s<=1}
begin
  if IsNanOrInf(s) or IsNanOrInf(x) or (s<0.0) or (abs(x) > 1.0) then begin
    {$ifopt R+}
      if RTE_ArgumentRange>0 then RunError(byte(RTE_ArgumentRange));
    {$endif}
    sfc_polylogr := NaN_x;
    exit;
  end;
  {Li_s(x) = x*Phi(x,s,1), see e.g. NIST[30], 25.14.3}
  {or http://functions.wolfram.com/10.08.26.0010.01}
  if x=0.0 then sfc_polylogr := 0.0
  else if s>=64.0 then sfc_polylogr := x
  else begin
    if s>=40.5 then sfc_polylogr := x*(1.0 + x*exp2(-s))
    else begin
      if frac(s)=0.0 then sfc_polylogr := sfc_polylog(trunc(s),x)
      else sfc_polylogr := x*sfc_lerch(x,s,1.0);
    end;
  end
end;


{---------------------------------------------------------------------------}
function fd_asymp_exp_int(n: integer; x: extended): extended;
  {-Fermi-Dirac asymptotic expansion for integer order, n>0, x > ln_MaxExt}
var
  p,s,t,f,z: extended;
  k: integer;
begin
  {Asymptotic expansion Goano [47], (9)}
  if n > x - 10 then begin
    {The AE is valid only if x >> n. The loop below converges e.g. for}
    {n = trunc(ln_MaxExt), x = ln_MaxExt+10, but the result is Inf. So}
    {return PosInf_x in this invalid situation for all these cases.}
    fd_asymp_exp_int := PosInf_x;
    exit;
  end;
  s := 1.0;
  p := 1.0;
  k := 0;
  z := sqr(1.0/x);
  f := 2.0*n*(n+1);
  repeat
    {loop entry invariants: p,f,s,t >= 0;  p = pochhammer(1-n,k)}
    f := f*z;
    inc(k,2);
    t := p*sfc_etaint(k)*f;
    s := s+t;
    p := p*(k-n-1)*(k-n);
  until t < eps_x*s;
  f := sfc_taylor(x,n+1);
  fd_asymp_exp_int := f*s;
end;


{---------------------------------------------------------------------------}
function fd_polylog_invf(n: integer; x,z: extended): extended;
  {-Return polylog(n,x), x<-1, n>2, z = ln(-x)}
var
  p,s,t: extended;
  j,k: integer;
begin
  {$ifdef debug}
    if (x >= -1) or (n <2) then RunError(byte(RTE_ArgumentRange));
  {$endif}
  {This is part of the standard polylog code, the only difference is,}
  {that z = ln(-x) is externally supplied to increase accuracy}
  s := 0.0;
  for k:=1 to n div 2 do begin
    j := 2*k;
    p := -sfc_etaint(j);
    j := n-j;
    if j>0 then p := sfc_taylor(z,j)*p;
    s := s + p;
  end;
  t := sfc_polylog(n, 1.0/x);
  if odd(n) then t := -t;
  fd_polylog_invf := 2.0*s - t - sfc_taylor(z,n);
end;


{---------------------------------------------------------------------------}
function sfc_fermi_dirac(n: integer; x: extended): extended;
  {-Return the integer order Fermi-Dirac integral F_n(x) = 1/n!*integral(t^n/(exp(t-x)+1), t=0..INF)}
var
  z: extended;
begin
  {Basic formula is F_n(x) = -polylog(n+1,-exp(x)}
  if x < -ln_MaxExt then begin
    sfc_fermi_dirac := 0.0;
  end
  else if x > ln_MaxExt then begin
    {asymptotic Goano [47], (9)}
    if n>0 then sfc_fermi_dirac := fd_asymp_exp_int(n, x)
    else case n of
        0: sfc_fermi_dirac := x;     {ln(1+e^x)   -> x}
       -1: sfc_fermi_dirac := 1.0;   {e^x/(1+e^x) -> 1}
      else sfc_fermi_dirac := 0.0;
    end;
  end
  else begin
    z := -exp(x);
    n := n+1;
    if z<-1.0 then begin
      {case x>0}
      if n<0 then begin
        {Li(-n,z) = (-1)^(n-1) * Li(-n,1/z}
        z := sfc_polylog(n, 1.0/z);
        if odd(n) then z := -z;
        sfc_fermi_dirac := z;
      end
      else if n>2 then sfc_fermi_dirac := -fd_polylog_invf(n, z, x)
      else begin
        {n=0,1,2: sfc_polylog can handle full range}
        sfc_fermi_dirac := -sfc_polylog(n, z);
      end;
    end
    else begin
      { -1 <= z < 0}
      sfc_fermi_dirac := -sfc_polylog(n, z);
    end;
  end;
end;


{---------------------------------------------------------------------------}
function fd_asymp_exp_real(q, x: extended): extended;
  {-Fermi-Dirac asymptotic expansion for real order q with q > -1, x >> q}
var
  p,s,t,r,f,z: extended;
  k: integer;
begin
  {Asymptotic expansion Goano [47], (9)}
  {$ifdef debug}
    if q + 10 > x then begin
      {The AE is valid only if x >> q. Normally used only}
      {for q=-1/2, 1/2, (3/2, 5/2) and x > ~40}
      fd_asymp_exp_real := PosInf_x;
      exit;
    end;
  {$endif}
  s := 1.0;
  p := 1.0;
  k := 0;
  z := sqr(1.0/x);
  f := 2.0*q*(q+1.0);
  t := f*1e20;
  repeat
    {p = pochhammer(1-n,k)}
    f := f*z;
    inc(k,2);
    r := p*sfc_etaint(k)*f;
    if abs(r) <= abs(t) then begin
      {terms of AE are still decreasing}
      t := r;
      s := s+t;
      p := p*((k-1)-q)*(k-q);
    end
    else t := 0.0;
  until abs(t) <= eps_x*abs(s);
  f := power(x,q+1.0)*sfc_rgamma(q+2.0);
  fd_asymp_exp_real := f*s;
end;


{---------------------------------------------------------------------------}
function sfc_fdp05(x: extended): extended;
  {-Return the complete Fermi-Dirac integral F(1/2,x)}
var
  t: extended;

{Note: THexExtW are needed with 16-bit compilers for relative errors}
{less than 5 eps_x, they are not needed for the 32+ bit compilers.}
const
  { t05 := t -> fd(1/2, 12*(t+2))/(12*(t+2))^(3/2);}
  { x=12*(t+2), x=12..36, t=x/12-2                 }
  { chebyshev(t05(t),t=-1..1,1e-20);               }
  np05_2 = 35;
  fdp05x2: array[0..np05_2 - 1] of THexExtW = (
             ($0325,$93D4,$D6AE,$C136,$3FFF),  {2*0.754743020650337869444270498780,    }
             ($1332,$AD56,$7AC6,$A38D,$BFF6),  { -0.249561545813512079811039605579e-2, }
             ($31E0,$2E66,$52CB,$80BA,$3FF5),  { +0.982115370416634900398147501155e-3, }
             ($A8A0,$B79D,$9E81,$B671,$BFF3),  { -0.347983979931683442238436985807e-3, }
             ($6CAC,$2A13,$2F9A,$F3D6,$3FF1),  { +0.116270381856917279446003317185e-3, }
             ($F249,$CDCD,$408E,$9CFC,$BFF0),  { -0.374282264886563175294182249759e-4, }
             ($547E,$54BC,$A0FE,$C50C,$3FEE),  { +0.117450554111933012129375923807e-4, }
             ($11DD,$D90C,$0D37,$F2DA,$BFEC),  { -0.361877328445089611740599641640e-5, }
             ($B3E1,$433A,$A033,$93A2,$3FEB),  { +0.109996838094100301284634122410e-5, }
             ($3117,$E381,$EF27,$B1AA,$BFE9),  { -0.330931901425378656153557493753e-6, }
             ($F14C,$5DFF,$6DFD,$D41D,$3FE7),  { +0.987737251258776378533216391827e-7, }
             ($4DFD,$B405,$59FB,$FBA2,$BFE5),  { -0.292940746917047935446228969668e-7, }
             ($91D1,$55E6,$C7EC,$9475,$3FE4),  { +0.864151410267920646768938404232e-8, }
             ($3CFE,$C040,$5C3C,$AE54,$BFE2),  { -0.253682857771686895946534432944e-8, }
             ($FB73,$7874,$C1CD,$CBBB,$3FE0),  { +0.741177885877868223911943372712e-9, }
             ($6DCD,$715E,$0C92,$ECE9,$BFDE),  { -0.215468706377707924304414487698e-9, }
             ($494B,$801D,$C632,$88FF,$3FDD),  { +0.622999859844165129697643004086e-10,}
             ($38FE,$2B65,$BC9B,$9D81,$BFDB),  { -0.179064482098473796784605475808e-10,}
             ($23F5,$0871,$71CC,$B3EC,$3FD9),  { +0.511373660300697402133999999999e-11,}
             ($11E2,$C31F,$D8E2,$CC23,$BFD7),  { -0.145050214090160134093432434719e-11,}
             ($1915,$6061,$2A01,$E602,$3FD5),  { +0.408577089407887189170064356955e-12,}
             ($D795,$E7C5,$F123,$80AE,$BFD4),  { -0.114293789551114639267975259137e-12,}
             ($3BA0,$F4DF,$2AFC,$8F08,$3FD2),  { +0.317594630475703386901919669155e-13,}
             ($7CE1,$4717,$D5D7,$9DFD,$BFD0),  { -0.877029250354717665156906845619e-14,}
             ($289F,$725E,$A63D,$AD87,$3FCE),  { +0.240821085811104890886594570029e-14,}
             ($EBF1,$CB47,$2005,$BDA5,$BFCC),  { -0.657963336164063747675726341907e-15,}
             ($FE35,$40C0,$B51C,$CE5D,$3FCA),  { +0.178994011257686697681940768666e-15,}
             ($FA95,$B066,$076E,$DFC0,$BFC8),  { -0.485180718104061909634313862435e-16,}
             ($A0FE,$B3A4,$A3F0,$F1E0,$3FC6),  { +0.131122056316856541233113653119e-16,}
             ($7B28,$4D87,$4BC6,$826C,$BFC5),  { -0.353512334501432389833325039827e-17,}
             ($D3DF,$68EE,$19DC,$8C62,$3FC3),  { +0.951273613288044223998852213460e-18,}
             ($D286,$C4CB,$1F59,$96E1,$BFC1),  { -0.255599619674429801332567738120e-18,}
             ($81DB,$D217,$D5FA,$A1F9,$3FBF),  { +0.685994710108645567896702114342e-19,}
             ($30D7,$33A1,$EDF0,$ADBD,$BFBD),  { -0.183956404622969457008106663899e-19,}
             ($2063,$51A5,$A578,$BA40,$3FBB)); { +0.493006328361511158404496844422e-20)}
var
  fdp05_2: array[0..np05_2 - 1] of extended absolute fdp05x2;

const
  { t01 := t -> fd(1/2, 4*(t+2));    }
  { x = 4*(t+2), x=4..12, t = x/4-2  }
  { chebyshev(t01(t),t=-1..1,1e-20); }
  np05_1 = 29;
  fdp05x1: array[0..np05_1 - 1] of THexExtW = (
             ($9036,$2A89,$1E59,$917C,$4004),  {2*18.1856047597986575196785666713,      }
             ($CE1E,$584A,$A0C0,$C8E6,$4002),  { +12.5563056481811830249567824342,      }
             ($48DB,$74EC,$C332,$D5D7,$3FFE),  {  +0.835323524302041720793490203916,    }
             ($350E,$7608,$78A3,$A8F2,$BFFA),  {  -0.412468635980776559531897297026e-1, }
             ($8CAF,$FBFB,$F802,$9ED8,$3FF7),  {  +0.484764203972064404543231999918e-2, }
             ($9E82,$FEC5,$6318,$BEEF,$BFF4),  {  -0.728359626235761181607282375542e-3, }
             ($D277,$BBEA,$8373,$E53F,$3FF1),  {  +0.109314012312724226246017534127e-3, }
             ($D011,$0B98,$04F1,$DAEC,$BFEE),  {  -0.130487650872812128272229904414e-4, }
             ($95AE,$67EC,$7A01,$D140,$3FE9),  {  +0.389761965081220490099761304103e-6, }
             ($CB0D,$E245,$01BA,$E0BB,$3FE9),  {  +0.418593166671574474687704852229e-6, }
             ($3EEB,$A5AB,$8D3E,$BF62,$BFE8),  {  -0.178241140882781326674081586696e-6, }
             ($6752,$1E45,$A71B,$D759,$3FE6),  {  +0.501401270935653913653534971074e-7, }
             ($41C0,$19D3,$3F38,$C495,$BFE4),  {  -0.114426363665601208334168909836e-7, }
             ($D0DD,$F32C,$7F70,$96A1,$3FE2),  {  +0.219196737232486715284670447279e-8, }
             ($C520,$0180,$2F58,$B8C4,$BFDF),  {  -0.336088028068196988951049728505e-9, }
             ($63D6,$88E9,$6123,$87F0,$3FDC),  {  +0.309089460235051920170794233970e-10,}
             ($2A7F,$C55D,$0F79,$FD9B,$3FD8),  {  +0.360395382713747844307256979851e-11,}
             ($AB3A,$0FBA,$168C,$CF46,$BFD8),  {  -0.294553759575921784436627612442e-11,}
             ($AE44,$1830,$9490,$8F75,$3FD7),  {  +0.101933962170251609766666666667e-11,}
             ($05A1,$DC32,$D55A,$9820,$BFD5),  {  -0.270234067190522926130300324494e-12,}
             ($5190,$1535,$5E00,$86C3,$3FD3),  {  +0.598468621686930875020197021442e-13,}
             ($94D4,$AA55,$68D0,$C665,$BFD0),  {  -0.110131976095155602873469185472e-13,}
             ($3C2E,$FE60,$C044,$D9AC,$3FCD),  {  +0.151042240381539761772702788261e-14,}
             ($782A,$C38A,$7E28,$9173,$BFC9),  {  -0.630793784338055501351350020007e-16,}
             ($7617,$CA85,$75E8,$ED02,$BFC8),  {  -0.513932671709885284753382122899e-16,}
             ($7515,$CA08,$2EE7,$E368,$3FC7),  {  +0.246555126271970717149547742394e-16,}
             ($4387,$E419,$6D72,$8C9B,$BFC6),  {  -0.762232826802428016612044204458e-17,}
             ($57FC,$2AA3,$B920,$8C61,$3FC4),  {  +0.190252722217962867356929160937e-17,}
             ($CAB7,$C88B,$F3B5,$EB11,$BFC1)); {  -0.398224281531035871813174429148e-18)}
var
  fdp05_1: array[0..np05_1 - 1] of extended absolute fdp05x1;

const
  { t00 := t -> fd(1/2, 2*(t+1));    }
  { x = 2*(t+1), x=0..4, t = x/2 - 1 }
  { chebyshev(t00(t),t=-1..1,1e-20); }
  np05_0 = 31;
  fdp05x0: array[0..np05_0 - 1] of THexExtW = (
             ($5CAE,$FEB4,$A28E,$CEF8,$4001),  {2*3.23392547573422976421866059174,     }
             ($9337,$80EE,$7BC1,$B8A7,$4000),  { +2.88522237679328803577304079965,     }
             ($2700,$C1B3,$1A93,$D096,$3FFD),  { +0.407395201241206365841823162781,    }
             ($65B1,$69A3,$AD8A,$D352,$BFF8),  { -0.128981299145840220950575776539e-1, }
             ($A854,$62A5,$BBDD,$BDA4,$BFF6),  { -0.289373003523482132846797286683e-2, }
             ($4C64,$8FBE,$767C,$EDCC,$3FF4),  { +0.907129985662663210233804487629e-3, }
             ($9CA9,$41AF,$63E6,$A169,$BFF1),  { -0.769670870754619578482136591047e-4, }
             ($1225,$B186,$0662,$AC14,$BFEF),  { -0.205133226436761049829261853507e-4, }
             ($1C19,$1377,$110A,$FD78,$3FED),  { +0.755396515130974716466786294881e-5, }
             ($6962,$D882,$C194,$B060,$BFEA),  { -0.657059080221453744877807438359e-6, }
             ($AAAA,$E200,$EF49,$EBA7,$BFE8),  { -0.219471747991019243265527185292e-6, }
             ($8D00,$F29B,$ADEE,$AC92,$3FE7),  { +0.803605497409122797720816119775e-7, }
             ($BDA2,$B8FE,$A9FF,$E227,$BFE3),  { -0.658197496457748650644446730955e-8, }
             ($E6FA,$9CCC,$7935,$BAD5,$BFE2),  { -0.271879079459221500311994000407e-8, }
             ($D6FF,$5A41,$B672,$844F,$3FE1),  { +0.962691975768783000937454605612e-9, }
             ($65A7,$6AD4,$C992,$9D5F,$BFDD),  { -0.715654866752415373221538623120e-10,}
             ($8BC1,$0A9B,$D68A,$A082,$BFDC),  { -0.364959955979361731397687347572e-10,}
             ($8D1A,$4AE9,$39D5,$D9C1,$3FDA),  { +0.123779266638284628266366968565e-10,}
             ($3F3C,$4BDF,$E79D,$E3E2,$BFD6),  { -0.809614940976175061407222222221e-12,}
             ($DEB2,$2C38,$B7FC,$9110,$BFD6),  { -0.515375501880531288979262837777e-12,}
             ($A6C7,$AEF0,$E86C,$BBB5,$3FD4),  { +0.166720484322176377906478380684e-12,}
             ($D5BD,$0F8E,$5A42,$A6D7,$BFD0),  { -0.926154824934769235381753853068e-14,}
             ($E767,$3BCB,$E74E,$87B5,$BFD0),  { -0.753344945832780041274713386623e-14,}
             ($1876,$90E5,$FF34,$A728,$3FCE),  { +0.231981301008194528825032891928e-14,}
             ($E9BB,$329F,$4C76,$F07A,$BFC9),  { -0.104290590578891200329403020712e-15,}
             ($8C77,$1995,$9394,$8224,$BFCA),  { -0.112880951868285154135078466421e-15,}
             ($A00C,$6101,$C59A,$986F,$3FC8),  { +0.330544205143281718828522986914e-16,}
             ($FD33,$8938,$9038,$A45A,$BFC3),  { -0.111370441904753970375841475032e-17,}
             ($5A2A,$F971,$8A77,$FE41,$BFC3),  { -0.172290580181768367727739031180e-17,}
             ($C00D,$5C97,$3D12,$8D83,$3FC2),  { +0.479463510149535766603385225170e-18,}
             ($426C,$D991,$2518,$FBE7,$BFBD)); { -0.266712580144022466545676300702e-19)}
var
  fdp05_0: array[0..np05_0 - 1] of extended absolute fdp05x0;
begin
  if x < ln_MinExt then sfc_fdp05 := 0.0
  else if x<=0.0 then begin
    t := -exp(x);
    sfc_fdp05 := -sfc_polylogr(1.5,t);
  end
  else if x<=4.0 then begin
    t := 0.5*x - 1.0;
    sfc_fdp05 := CSEvalX(t,fdp05_0,np05_0);
  end
  else if x<=12.0 then begin
    t := 0.25*x - 2.0;
    sfc_fdp05 := CSEvalX(t,fdp05_1,np05_1);
  end
  else if x<=36.0 then begin
    t := x/12.0 - 2.0;
    sfc_fdp05 := CSEvalX(t,fdp05_2,np05_2)*(x*sqrt(x));
  end
  else sfc_fdp05 := fd_asymp_exp_real(0.5, x);
end;


{---------------------------------------------------------------------------}
function sfc_fdp15(x: extended): extended;
  {-Return the complete Fermi-Dirac integral F(3/2,x)}
var
  t: extended;
{Note: THexExtW are needed with 16-bit compilers for relative errors}
{less than 5 eps_x, they are not needed for the 32+ bit compilers.}
const
  { t05 := t -> fd(3/2, 12*(t+2))/(12*(t+2))^(1/2);}
  { x=12*(t+2), x=12..36, t=x/12-2                 }
  { chebyshev(t05(t),t=-1..1,1e-20);               }
  np15_2 = 29;
  fdp15x2: array[0..np15_2 - 1] of THexExtW = (
             ($E207,$4F91,$ACD8,$C4D6,$4007),  {+3.9367714980969664415E+2}
             ($CEB6,$2A9E,$0CEA,$AD52,$4006),  {+1.7332050956287894902E+2}
             ($5BA4,$DC89,$79BE,$AD50,$4003),  {+2.1664294711213342222E+1}
             ($05BA,$478F,$2C2C,$DCF3,$3FF2),  {+2.1071423795317094221E-4}
             ($EE34,$17C2,$FCB3,$96AD,$BFF1),  {-7.1849649972728518409E-5}
             ($B36B,$4181,$FEBC,$C6FB,$3FEF),  {+2.3720783675115948540E-5}
             ($0519,$6F14,$C84A,$80C7,$BFEE),  {-7.6759099927198066013E-6}
             ($88B8,$DBD0,$582E,$A496,$3FEC),  {+2.4525416354303170927E-6}
             ($EE2C,$E0A9,$7415,$D092,$BFEA),  {-7.7699156039780901828E-7}
             ($A360,$CFB5,$A1B1,$8344,$3FE9),  {+2.4450587530326532190E-7}
             ($E27B,$7E6F,$607A,$A40F,$BFE7),  {-7.6396421480437313314E-8}
             ($336B,$BC60,$2D36,$CB27,$3FE5),  {+2.3650125793416319129E-8}
             ($770C,$CF48,$8756,$F861,$BFE3),  {-7.2288376791631086374E-9}
             ($2356,$3897,$8029,$9549,$3FE2),  {+2.1724133960664972412E-9}
             ($EBB9,$0777,$8582,$AFA5,$BFE0),  {-6.3899849344775435431E-10}
             ($D9A7,$825E,$AC5B,$C960,$3FDE),  {+1.8315188750975328184E-10}
             ($6462,$5940,$BE83,$E005,$BFDC),  {-5.0936805166409347329E-11}
             ($B67D,$1185,$6C51,$F0D0,$3FDA),  {+1.3688699754156613657E-11}
             ($138E,$20AF,$FAEF,$F90D,$BFD8),  {-3.5392788817089458740E-12}
             ($2D8A,$5F9D,$AE78,$F689,$3FD6),  {+8.7587827996357287805E-13}
             ($FE62,$A84D,$4B0B,$E7EC,$BFD4),  {-2.0598902148806642654E-13}
             ($4EC6,$5EB9,$BC82,$CCF7,$3FD2),  {+4.5511976448198640562E-14}
             ($46CB,$7098,$B707,$A690,$BFD0),  {-9.2462311586020405740E-15}
             ($745C,$0AA6,$080C,$ED2A,$3FCD),  {+1.6456571195203601819E-15}
             ($1D65,$D535,$8794,$FE27,$BFCA),  {-2.2044381299342686014E-16}
             ($A1ED,$43FC,$DCDE,$86CE,$3FC5),  {+3.6539797251750199626E-18}
             ($61A1,$472A,$2005,$E2FD,$3FC6),  {+1.2305085869833743187E-17}
             ($B276,$0238,$0DEB,$EA41,$BFC5),  {-6.3494706080876196549E-18}
             ($59BF,$ED88,$52F5,$AF53,$3FC4)); {+2.3761033915874246600E-18}
var
  fdp15_2: array[0..np15_2 - 1] of extended absolute fdp15x2;

const
  { t01 := t -> fd(3/2, 4*(t+2));    }
  { x = 4*(t+2), x=4..12, t = x/4-2  }
  { chebyshev(t01(t),t=-1..1,1e-20); }
  np15_1 = 28;
  fdp15x1: array[0..np15_1 - 1] of THexExtW = (
             ($D0FD,$CE4A,$1F4F,$909F,$4006),  {+1.4462157152925903243E+2}
             ($DF13,$60B5,$BF4C,$8E24,$4005),  {+7.1071771990590546640E+1}
             ($D653,$FBC0,$9338,$C98F,$4002),  {+1.2597552511779260681E+1}
             ($8BD7,$9F4D,$0B81,$8DBC,$3FFE),  {+5.5365058817488071785E-1}
             ($1E94,$120D,$BB17,$A5F6,$BFF9),  {-2.0259251985920947386E-2}
             ($9BD4,$E1AC,$C321,$F86C,$3FF5),  {+1.8953312109631679276E-3}
             ($4F03,$A374,$EEB1,$FA04,$BFF2),  {-2.3843695371615998960E-4}
             ($3A38,$8F26,$2645,$8288,$3FF0),  {+3.1121214385040858788E-5}
             ($FE69,$E2AA,$DCFE,$E1F1,$BFEC),  {-3.3668395634881968254E-6}
             ($5082,$A856,$E3D5,$8787,$3FE8),  {+1.2622291243644484818E-7}
             ($4B4F,$CB97,$D712,$9E3F,$3FE7),  {+7.3690607915601816666E-8}
             ($D950,$8E95,$E3D9,$8240,$BFE6),  {-3.0327000821131128333E-8}
             ($BC2E,$5F61,$5F6D,$894A,$3FE4),  {+7.9913599535400873690E-9}
             ($0E44,$6E7F,$8710,$EAD7,$BFE1),  {-1.7086997443833728992E-9}
             ($9F34,$2A53,$D90E,$A9B8,$3FDF),  {+3.0872263232876599441E-10}
             ($54C1,$C11D,$8E5C,$C732,$BFDC),  {-4.5292264252711262319E-11}
             ($0E8A,$49E5,$C28C,$94E4,$3FD9),  {+4.2318104524080512329E-12}
             ($2149,$EA9B,$6E4C,$AB2D,$3FD5),  {+3.0407225946293674652E-13}
             ($5EB2,$BCA0,$18C7,$A757,$BFD5),  {-2.9725594761874387986E-13}
             ($6DAD,$C831,$18BC,$E36E,$3FD3),  {+1.0099923784566557989E-13}
             ($2829,$BE32,$4353,$E97C,$BFD1),  {-2.5922086958100736660E-14}
             ($1901,$6F98,$AF3B,$C82B,$3FCF),  {+5.5558514061788275762E-15}
             ($2F0C,$4096,$2FE0,$8F76,$BFCD),  {-9.9546529373470499049E-16}
             ($F65C,$93F7,$0DE1,$9C94,$3FCA),  {+1.3581005834664229560E-16}
             ($C873,$795A,$5BEC,$86DE,$BFC6),  {-7.3112409217500735597E-18}
             ($1264,$00F5,$62E9,$8130,$BFC5),  {-3.5016751122371363843E-18}
             ($E86D,$655C,$E73D,$8124,$3FC4),  {+1.7502296465398255808E-18}
             ($2BBE,$9519,$6F35,$9DF0,$BFC2)); {-5.3511881381428642648E-19}
var
  fdp15_1: array[0..np15_1 - 1] of extended absolute fdp15x1;

const
  { t00 := t -> fd(3/2, 2*(t+1));    }
  { x = 2*(t+1), x=0..4, t = x/2 - 1 }
  { chebyshev(t00(t),t=-1..1,1e-20); }
  np15_0 = 30;
  fdp15x0: array[0..np15_0 - 1] of THexExtW = (
             ($AA8A,$8D18,$5613,$B3C7,$4002),  {+1.1236166073199948765E+1}
             ($2A3E,$C299,$40E5,$C1EF,$4001),  {+6.0604557502272531624}
             ($369D,$0B58,$CE6F,$B97A,$3FFF),  {+1.4490602533539360290}
             ($4C36,$A850,$9807,$8C0B,$3FFC),  {+1.3676297709214706240E-1}
             ($4A77,$329F,$74F2,$E22F,$BFF6),  {-3.4513149750616713262E-3}
             ($55F2,$5A13,$F3CB,$93AD,$BFF4),  {-5.6335258963187187413E-4}
             ($FDF9,$C386,$0F1F,$A21E,$3FF2),  {+1.5460721805105655254E-4}
             ($350D,$0351,$4ED1,$CA93,$BFEE),  {-1.2074436032395957859E-5}
             ($FEDA,$0AC1,$0056,$A691,$BFEC),  {-2.4820329454318314047E-6}
             ($1DB7,$9078,$0EAE,$E7DA,$3FEA),  {+8.6371521103341848976E-7}
             ($AF35,$9244,$12A8,$9E5C,$BFE7),  {-7.3741962996236602468E-8}
             ($54E6,$D486,$0D12,$A63F,$BFE5),  {-1.9353615729676523342E-8}
             ($9AF5,$2A02,$CCF6,$EDE1,$3FE3),  {+6.9232783779587078981E-9}
             ($8215,$CE7F,$2238,$9F87,$BFE0),  {-5.8035899541125150058E-10}
             ($BB33,$C1AB,$67E5,$CFE7,$BFDE),  {-1.8908752199406953327E-10}
             ($F285,$93D3,$857E,$927B,$3FDD),  {+6.6612531424447944938E-11}
             ($974B,$1431,$F0CD,$B897,$BFD9),  {-5.2464633336918750095E-12}
             ($4A22,$E6FC,$CE0A,$93B7,$BFD8),  {-2.0991988621741175341E-12}
             ($3476,$7DEF,$7684,$C99E,$3FD6),  {+7.1629456476161078420E-13}
             ($0782,$7232,$1C65,$E76C,$BFD2),  {-5.1386075015702707325E-14}
             ($9EF4,$D32A,$F751,$E3EE,$BFD1),  {-2.5305697681559179825E-14}
             ($EBA8,$DD77,$E0AB,$957A,$3FD0),  {+8.2978063705001989763E-15}
             ($9F96,$3CEE,$58C5,$97BB,$BFCC),  {-5.2642551179225625266E-16}
             ($D1B4,$208E,$67F1,$BA33,$BFCB),  {-3.2300690729343082800E-16}
             ($459C,$C353,$B5E7,$E9B9,$3FC9),  {+1.0136224841459294120E-16}
             ($5530,$915C,$7A68,$CAAF,$BFC5),  {-5.4938004437287676996E-18}
             ($5440,$9452,$743F,$9E98,$BFC5),  {-4.2987402865091326124E-18}
             ($214C,$693F,$1EBB,$BE15,$3FC3),  {+1.2880491228202175925E-18}
             ($B48E,$1FC3,$3F4A,$8659,$BFBF),  {-5.6898854614185145094E-20}
             ($1422,$0469,$F0EC,$8B6D,$BFBF)); {-5.9050672129271432028E-20}
var
  fdp15_0: array[0..np15_0 - 1] of extended absolute fdp15x0;
begin
  if x < ln_MinExt then sfc_fdp15 := 0.0
  else if x<=0.0 then begin
    t := -exp(x);
    sfc_fdp15 := -sfc_polylogr(2.5,t);
  end
  else if x<=4.0 then begin
    t := 0.5*x - 1.0;
    sfc_fdp15 := CSEvalX(t,fdp15_0,np15_0);
  end
  else if x<=12.0 then begin
    t := 0.25*x - 2.0;
    sfc_fdp15 := CSEvalX(t,fdp15_1,np15_1);
  end
  else if x<=36.0 then begin
    t := x/12.0 - 2.0;
    sfc_fdp15 := CSEvalX(t,fdp15_2,np15_2)*sqrt(x);
  end
  else sfc_fdp15 := fd_asymp_exp_real(1.5, x);
end;


{---------------------------------------------------------------------------}
function sfc_fdm05(x: extended): extended;
  {-Return the complete Fermi-Dirac integral F(-1/2,x)}
var
  t: extended;
{Note: THexExtW are needed with 16-bit compilers for relative errors}
{less than 5 eps_x, they are not needed for the 32+ bit compilers.}
const
  { t05 := t -> fd(-1/2, 14*(t+2)-2)/(14*(t+2)-2)^(1/2); }
  { x = 14*(t+2)-2, x=12..40, t = (x+2)/14-2             }
  { chebyshev(t05(t),t=-1..1,1e-20);                     }
  nm05_2 = 36;
  fdm05x2: array[0..nm05_2 - 1] of THexExtW = (
             ($39CB,$588B,$5F29,$9048,$4000),  {2*1.12720860976068042985126031469,      }
             ($EF98,$4015,$5936,$A6EC,$3FF5),  { +0.127352322590483468748105626469e-2,  }
             ($C2CB,$28D0,$8398,$9058,$BFF4),  { -0.550635368235688347038739063159e-3,  }
             ($CCBD,$142D,$349D,$E222,$3FF2),  { +0.215657821006449458356062287107e-3,  }
             ($187E,$C82E,$1C87,$A7E5,$BFF1),  { -0.800585587450641857019601765123e-4,  }
             ($18D4,$F791,$E75B,$F170,$3FEF),  { +0.287820136859342153053116112911e-4,  }
             ($4E41,$9762,$7838,$AA28,$BFEE),  { -0.101422121779796149201023928609e-4,  }
             ($1709,$3850,$27A3,$ECD1,$3FEC),  { +0.352884845542076071119669637285e-5,  }
             ($0C55,$0821,$754B,$A371,$BFEB),  { -0.121774670482100079163136672498e-5,  }
             ($C921,$FF00,$DD5E,$E042,$3FE9),  { +0.417719018374740984434001320844e-6,  }
             ($A682,$1C8B,$B733,$98FD,$BFE8),  { -0.142484043403418353187352871031e-6,  }
             ($3B14,$CB5D,$1979,$CF4B,$3FE6),  { +0.482642458470626719951342032248e-7,  }
             ($DCA7,$DE58,$F41C,$8B15,$BFE5),  { -0.161917130596971286058653227519e-7,  }
             ($FC0E,$2385,$3937,$B82D,$3FE3),  { +0.536024612047194258605653270535e-8,  }
             ($097A,$2926,$6F3E,$EFA9,$BFE1),  { -0.174376948911707269461907228582e-8,  }
             ($2D95,$9F93,$7ED2,$9890,$3FE0),  { +0.555026181833641050021767137440e-9,  }
             ($A662,$6548,$2EF3,$BD38,$BFDE),  { -0.172094102173839496709895590169e-9,  }
             ($B081,$C2C2,$B0CD,$E39E,$3FDC),  { +0.517547699271684434423133548903e-10, }
             ($9E8D,$458C,$0C22,$842D,$BFDB),  { -0.150266676426121966101287037037e-10, }
             ($6206,$B5EE,$3062,$9367,$3FD9),  { +0.418944756732711716944423047911e-11, }
             ($A9CA,$26C7,$AEBD,$9CB5,$BFD7),  { -0.111348937219875590656629403579e-11, }
             ($65E6,$3122,$2843,$9D0C,$3FD5),  { +0.278972381829081126481744940963e-12, }
             ($8286,$BBFF,$3CB9,$915D,$BFD3),  { -0.645546762036873772076350520773e-13, }
             ($954D,$E342,$5E99,$EDBA,$3FD0),  { +0.131965552928934103834300756587e-13, }
             ($D4D8,$E7DE,$0CA9,$9669,$BFCE),  { -0.208736291421929647980739225035e-14, }
             ($4E6F,$4C39,$B018,$C962,$3FC9),  { +0.873370384238809757090654794064e-16, }
             ($A9DA,$3F4D,$0930,$8C9A,$3FCA),  { +0.121952537222219205275296984895e-15, }
             ($301F,$34B4,$0170,$AE73,$BFC9),  { -0.756552983007583718512206829240e-16, }
             ($831A,$E68D,$DA2D,$9665,$3FC8),  { +0.326123373969832726648499559418e-16, }
             ($37C7,$E169,$55EF,$E141,$BFC6),  { -0.122111098113026111719660711879e-16, }
             ($9929,$EEEF,$5F4B,$9C4F,$3FC5),  { +0.423679233680363253635062387618e-17, }
             ($6278,$BFF2,$85BF,$CECF,$BFC3),  { -0.140140337077744622807440276665e-17, }
             ($8304,$2C42,$89BC,$846B,$3FC2),  { +0.448656650094168829272055386020e-18, }
             ($B535,$0E51,$F2CA,$A599,$BFC0),  { -0.140269808809750966318869781429e-18, }
             ($0C9F,$83C5,$0325,$CB59,$3FBE),  { +0.430605513146981698934338690424e-19, }
             ($CFAB,$2EFA,$877A,$F5FB,$BFBC)); { -0.130222070259439058246940543349e-19, }
var
  fdm05_2: array[0..nm05_2 - 1] of extended absolute fdm05x2;

const
  { t01 := t -> fd(-1/2, 4*(t+2));   }
  { x = 4*(t+2), x=4..12, t = x/4-2  }
  { chebyshev(t01(t),t=-1..1,1e-20); }
  nm05_1 = 30;
  fdm05x1: array[0..nm05_1 - 1] of THexExtW = (
             ($81A4,$8736,$836A,$C6DC,$4001),  {2*3.10720906642095693549141663063,     }
             ($45F7,$5FCA,$AF0A,$D868,$3FFE),  { +0.845347347290762745698264830651,    }
             ($1E8C,$4513,$5574,$8287,$BFFB),  { -0.637346912486776414955579558212e-1, }
             ($46F4,$B77F,$F5FA,$A43A,$3FF8),  { +0.100238229887210249047746267365e-1, }
             ($B06D,$233D,$BE6B,$F45E,$BFF5),  { -0.186439585156115756577336126739e-2, }
             ($48AE,$7077,$BEF7,$AC3F,$3FF3),  { +0.328538909279736813909986738098e-3, }
             ($4941,$98D5,$517C,$B670,$BFF0),  { -0.434967859717546117551553285448e-4, }
             ($A907,$0E55,$C147,$A038,$3FEA),  { +0.596872341564135171934135718907e-6, }
             ($CCE1,$1506,$2D65,$91E3,$3FEC),  { +0.217389183372963314012513808309e-5, }
             ($C12B,$E0C1,$195D,$8124,$BFEB),  { -0.962175518760746788464909336684e-6, }
             ($554C,$ADFC,$E35F,$9BCF,$3FE9),  { +0.290222583707548004030466363153e-6, }
             ($9D7E,$936B,$116B,$9868,$BFE7),  { -0.709698143468401550945013829553e-7, }
             ($C85D,$2593,$0FA1,$F848,$3FE4),  { +0.144518846929383515210221101027e-7, }
             ($85AC,$01A5,$4030,$9F04,$BFE2),  { -0.231399614747943009400043966811e-8, }
             ($3DA3,$9ABD,$2266,$E068,$3FDE),  { +0.204096772826715027518323729570e-9, }
             ($B411,$EAC9,$4F3F,$A9DA,$3FDC),  { +0.386200489979488286568147019637e-10,}
             ($EC72,$2FC2,$9DE0,$F3D4,$BFDB),  { -0.277203223495739126097345324431e-10,}
             ($7B46,$206C,$1E0C,$AC33,$3FDA),  { +0.978841838084900111240803825558e-11,}
             ($B41F,$F864,$2F5A,$BCD1,$BFD8),  { -0.268325278562056093253240740740e-11,}
             ($739A,$535B,$6EA0,$ACED,$3FD6),  { +0.614361785526356233322403621196e-12,}
             ($D6FF,$2E94,$1FF1,$82A3,$BFD4),  { -0.116029147310593134362697691583e-12,}
             ($B4A5,$1B19,$2407,$8F27,$3FD1),  { +0.158931638394253582586663743109e-13,}
             ($AC3A,$995A,$507D,$E126,$BFCB),  { -0.390572410679751374699284640444e-15,}
             ($9A87,$790E,$050A,$CFF4,$BFCC),  { -0.721482602544015576484549245755e-15,}
             ($6DE8,$98D3,$BA37,$C105,$3FCB),  { +0.334840441309012455905148106174e-15,}
             ($11C8,$0955,$64DF,$F191,$BFC9),  { -0.104763396492153632437572704125e-15,}
             ($2E77,$2FCC,$5928,$F5C5,$3FC7),  { +0.266465334690489637299281537629e-16,}
             ($64F7,$6AC2,$3045,$D14D,$BFC5),  { -0.567312900783804305906703267603e-17,}
             ($658A,$7DA0,$0C1E,$8E07,$3FC3),  { +0.962415969624123143993085069845e-18,}
             ($728A,$D261,$C196,$E75E,$BFBF)); { -0.979890664035765026812190968018e-19)}
var
  fdm05_1: array[0..nm05_1 - 1] of extended absolute fdm05x1;

const
  { t00 := t -> fd(-1/2, 2*(t+1));   }
  { x = 2*(t+1), x=0..4, t = x/2 - 1 }
  { chebyshev(t00(t),t=-1..1,1e-20); }
  nm05_0 = 33;
  fdm05x0: array[0..nm05_0 - 1] of THexExtW = (
             ($C466,$7A74,$6418,$B675,$4000),  {2*1.42545748896819736142428828509,     }
             ($F317,$B08B,$13D3,$CD85,$3FFE),  { +0.802811850721397311632302091077,    }
             ($B45B,$9E73,$EA41,$8C85,$BFFA),  { -0.343073988568933129244642294564e-1, }
             ($FA43,$49CC,$B004,$C441,$BFF8),  { -0.119785517610154200513442344857e-1, }
             ($BF4F,$8636,$BF31,$8FC0,$3FF7),  { +0.438699088685875336070850350529e-2, }
             ($3DF7,$E4EA,$84DC,$D39E,$BFF3),  { -0.403631620076134737472343018429e-3, }
             ($0DF4,$7406,$5B8A,$9BE1,$BFF2),  { -0.148659041454562690460518932844e-3, }
             ($683B,$ECE9,$87E3,$F3FC,$3FF0),  { +0.581709023766370096169389361999e-4, }
             ($C27D,$1617,$BE83,$A9FA,$BFED),  { -0.506578294882995558003563539688e-5, }
             ($3DDC,$68DB,$9262,$97B8,$BFEC),  { -0.226081883384096770040396740859e-5, }
             ($A089,$EBD7,$DA22,$E390,$3FEA),  { +0.847748773163128123864631556434e-6, }
             ($663E,$735A,$9A87,$8DF3,$BFE7),  { -0.661013539307752677486955427753e-7, }
             ($156A,$1BE4,$505A,$9B8D,$BFE6),  { -0.362172739869069536282661782116e-7, }
             ($3FE0,$BB25,$27C2,$DD51,$3FE4),  { +0.128823456441545703286380320048e-7, }
             ($6FB1,$1F66,$77A3,$EFF7,$BFE0),  { -0.872993657208158587706978751116e-9, }
             ($85E0,$2CDF,$7856,$A3A5,$BFE0),  { -0.595342016608391684486334590438e-9, }
             ($3BA3,$A39F,$893D,$DC70,$3FDE),  { +0.200488642920464472125344875726e-9, }
             ($87C0,$90FE,$7308,$C8A8,$BFDA),  { -0.114060870414129142501077323872e-10,}
             ($A47D,$BF8E,$434F,$AECC,$BFDA),  { -0.993611036461939592748999999999e-11,}
             ($7D8D,$9172,$4621,$DEDB,$3FD8),  { +0.316698189615823685519545765746e-11,}
             ($0253,$C2C7,$3512,$A21A,$BFD4),  { -0.143975828889301436878708068523e-12,}
             ($2EC9,$93A5,$C664,$BC81,$BFD4),  { -0.167427790285290702945465438147e-12,}
             ($D8D3,$C691,$C58F,$E381,$3FD2),  { +0.505166843470001025439981895705e-13,}
             ($4D48,$B668,$2C80,$F3D4,$BFCD),  { -0.169190220207909386709653151896e-14,}
             ($A558,$9B77,$94BC,$CC92,$BFCE),  { -0.283901488488463908424013117515e-14,}
             ($22DD,$2B0F,$8C61,$E9C6,$3FCC),  { +0.811071971814294943755263204708e-15,}
             ($CDC5,$BEFC,$1299,$9CB7,$BFC7),  { -0.169910881775102181849224823698e-16,}
             ($14B0,$2938,$4B9B,$DEF1,$BFC8),  { -0.483429615582375261924591357742e-16,}
             ($AF92,$5125,$818B,$F143,$3FC6),  { +0.130789311367733023108178561868e-16,}
             ($335A,$EAC4,$E45D,$EFE4,$BFBF),  { -0.101599107342478990354348239320e-18,}
             ($B0EF,$6AD3,$D25F,$F3A5,$BFC2),  { -0.825510657563228475743326075971e-18,}
             ($C04D,$6680,$FD3F,$F9AE,$3FC0),  { +0.211490194721918999612264245007e-18,}
             ($9607,$35A4,$9CEC,$8562,$BFBD)); { -0.141227081204025177342262136401e-19)}
var
  fdm05_0: array[0..nm05_0 - 1] of extended absolute fdm05x0;
begin
  if x < ln_MinExt then sfc_fdm05 := 0.0
  else if x<=0.0 then begin
    t := -exp(x);
    sfc_fdm05 := -sfc_polylogr(0.5,t);
  end
  else if x<=4.0 then begin
    t := 0.5*x - 1.0;
    sfc_fdm05 := CSEvalX(t,fdm05_0,nm05_0);
  end
  else if x<=12.0 then begin
    t := 0.25*x - 2.0;
    sfc_fdm05 := CSEvalX(t,fdm05_1,nm05_1);
  end
  else if x<=40.0 then begin
    t := (x+2.0)/14.0 - 2.0;
    sfc_fdm05 := CSEvalX(t,fdm05_2,nm05_2)*sqrt(x);
  end
  else sfc_fdm05 := fd_asymp_exp_real(-0.5, x);
end;


{---------------------------------------------------------------------------}
function sfc_ti2(x: extended): extended;
  {-Return the inverse tangent integral, ti2(x) = integral(arctan(t)/t, t=0..x)}
var
  y: extended;
const
  {chebyshev(int(arctan(t)/t, t=0..x)/x, x=-1..1, 0.5e-20); aics[0] doubled}
  nai = 23;
  aics: array[0..nai-1] of extended = (
          +0.191040361296235937512100085363e+1,
          -0.417635143765674693973184505426e-1,
          +0.275392550786367433987311337253e-2,
          -0.250518095262488814159890349742e-3,
          +0.266698128512117108101673214400e-4,
          -0.311890514107001322989208298185e-5,
          +0.388338531322492994856981431022e-6,
          -0.505727458496376372130362845503e-7,
          +0.681225282949264960525940397947e-8,
          -0.942125616543636974084406666666e-9,
          +0.133078788164081272284621690761e-9,
          -0.191267807507294407420492895005e-10,
          +0.278912620074783714512526059665e-11,
          -0.411748196101712508143369500279e-12,
          +0.614298719454016651500191474755e-13,
          -0.924928654021041585596995204818e-14,
          +0.140386740366392221748521257509e-14,
          -0.214598969089468659553281910650e-15,
          +0.330121857094999796296296296296e-16,
          -0.510717131686183481708845972024e-17,
          +0.794150680705224718491128609488e-18,
          -0.124060349865156550047951820204e-18,
          +0.194621210567853358154874940741e-19{,
          -0.306490766819307325157212222222e-20});
begin
  {ti2(x) = integral(arctan(t)/t,  t=0..x)}
  {ti2(x) = i/2*[dilog(-i*x) - dilog(i*x)]}
  {ti2(x) = ti2(1/x) + Pi/2*ln(x),  x<>0}

  {See references [22] (function atnint) and [21] (specfunc/atanint.c)}
  {Note that the precision of both refs are not suitable for extended!}

  y := abs(x);
  if y<3.41e-5 then begin
    {ti2(x) := x*(1 - 1/9x^2 + 1/25x^4 +O(x^6)}
    if y<6.98e-10 then sfc_ti2 := x
    else sfc_ti2 := x*(1.0-sqr(x)/9.0);
  end
  else begin
    {ti2(x)/x is even and it's Chebyshev approximation on [-1,1] contains }
    {only the even Chebyshev polynomials. Since T(2n,x) = T(n,2x^2-1), the}
    {coefficients aics can be used with CSEvalX and the argument 2x^2-1.  }
    if y<=1.0 then sfc_ti2 := x*CSEvalX(2.0*sqr(x)-1.0, aics, nai)
    else begin
      {ti2(x) = Pi/2*ln(x) + 1/x + O(1/x^2), x->Inf}
      if y>=3.04e9 then y := Pi_2*ln(y) + 1.0/y
      else begin
        {Use ti2(x) = ti2(1/x) + sign(x)*Pi/2*ln(|x|) for |x|>1}
        y := Pi_2*ln(y) + CSEvalX(2.0/sqr(x)-1.0, aics, nai)/y;
      end;
      if x>0.0 then sfc_ti2 := y else sfc_ti2 := -y;
    end;
  end;
end;


{---------------------------------------------------------------------------}
function sfc_pz(x: extended): extended;
  {-Return the prime zeta function P(x) = sum(1/p^x, p prime), x > 1}
var
  s,t,y: extended;
  k,m: integer;
begin
  if IsNan(x) or (x < 1.0) then begin
    {$ifopt R+}
      if RTE_ArgumentRange>0 then RunError(byte(RTE_ArgumentRange));
    {$endif}
    sfc_pz := NaN_x;
    exit;
  end;
  if x>=27.0 then begin
    {use only primes up to 7}
    if x>=16450.0 then sfc_pz := 0.0
    else if x>120.0 then sfc_pz := exp2(-x)
    else begin
      t := exp3(-x);
      if x<50.0 then begin
        s := exp5(-x);
        t := t + s;
        if x<36.0 then begin
          s := exp7(-x);
          t := t + s;
        end;
      end;
      s := exp2(-x);
      sfc_pz := s+t;
    end;
  end
  else if x=1.0 then sfc_pz := PosInf_x
  else begin
    {P(x) = sum(mu(k)/k*ln(zeta(kx)), k>0) by Moebius inversion, see e.g.}
    {H. Cohen, High Precision Computation of Hardy-Littlewood Constants, }
    {Section 2.1, from  http://www.math.u-bordeaux.fr/~cohen/hardylw.dvi }
    {With BASM Cohen's speed-up is used with A=7.}

    {$ifdef BASM}
      y := -x;
      s := exp2(y) + exp3(y) + exp5(y) + exp7(y);
    {$else}
      s := 0.0;
    {$endif}
    {maximum observed k is 19 with BASM, 61 without}
    for k:=1 to n_moeb do begin
      m := moebius[k];
      if m<>0 then begin
        y := k*x;
        t := ln1p(sfc_zetam1(y));
        {$ifdef BASM}
          t := t + ln1p(-exp2(-y));
          t := t + ln1p(-exp3(-y));
          t := t + ln1p(-exp5(-y));
          t := t + ln1p(-exp7(-y));
        {$endif}
        s := s + m*t/k;
        if t < eps_x*s then begin
          sfc_pz := s;
          exit;
        end;
      end;
    end;
    {No convergence}
    sfc_pz := s;
    if RTE_NoConvergence>0 then RunError(byte(RTE_NoConvergence));
  end;
end;


const
  ETAEPS = 1e-9;

{---------------------------------------------------------------------------}
function etam1pos(s: extended): extended;
  {-Return the eta(s)-1 = -sum((-1)^k/k^s, k=2..) for s >= -ETAEPS; internal use}
const
  dh: array[2..28] of THexExtW = (
        ($0000,$0000,$0000,$8000,$BFFF),   {-1.0000000000000000000}
        ($FF8D,$FFFF,$FFFF,$FFFF,$3FFE),   {+9.9999999999999999376E-1}
        ($9282,$FFFF,$FFFF,$FFFF,$BFFE),   {-9.9999999999999848049E-1}
        ($65DE,$FFD6,$FFFF,$FFFF,$3FFE),   {+9.9999999999985219972E-1}
        ($DB9D,$F790,$FFFF,$FFFF,$BFFE),   {-9.9999999999232918620E-1}
        ($5DE0,$F172,$FFFE,$FFFF,$3FFE),   {+9.9999999975393324796E-1}
        ($8E24,$1992,$FFE9,$FFFF,$BFFE),   {-9.9999999466815323230E-1}
        ($A850,$6906,$FE9C,$FFFF,$3FFE),   {+9.9999991720781145500E-1}
        ($D0ED,$ED7F,$EFE1,$FFFF,$BFFE),   {-9.9999903932393797886E-1}
        ($E353,$D679,$6FD9,$FFFF,$3FFE),   {+9.9999140804582214054E-1}
        ($96BA,$673E,$068A,$FFFC,$BFFE),   {-9.9993935469614779086E-1}
        ($2E1B,$5186,$7297,$FFE9,$3FFE),   {+9.9965587801263986058E-1}
        ($6897,$A3EA,$98D7,$FF97,$BFFE),   {-9.9840693726211216782E-1}
        ($6E55,$CC86,$EF58,$FE70,$3FFE),   {+9.9391075056021247386E-1}
        ($88D1,$90B3,$DCB1,$FB07,$BFFE),   {-9.8058871588791708418E-1}
        ($409B,$E7BC,$54F0,$F2AC,$3FFE),   {+9.4794207459902769260E-1}
        ($4AC1,$8818,$410E,$E1B0,$BFFE),   {-8.8159567456031699353E-1}
        ($8E11,$D7C2,$339A,$C50C,$3FFE),   {+7.6971743135778522652E-1}
        ($0478,$3AD9,$9AB8,$9D0A,$BFFE),   {-6.1344305990028053606E-1}
        ($FB33,$ED26,$5F81,$DDE4,$3FFD),   {+4.3338297329917271349E-1}
        ($C82A,$4CCD,$DDD9,$86E6,$BFFD),   {-2.6348012235248635784E-1}
        ($99BC,$14E9,$C9D0,$88D7,$3FFC),   {+1.3363566715745776083E-1}
        ($7EF0,$EDD4,$207E,$DF8B,$BFFA),   {-5.4576041161330412132E-2}
        ($7788,$4C41,$8AFB,$8C77,$3FF9),   {+1.7146846245289445112E-2}
        ($E28D,$A518,$4FE1,$FDE4,$BFF6),   {-3.8740820906649887224E-3}
        ($AE94,$D68C,$FF22,$926E,$3FF4),   {+5.5859977938736941181E-4}
        ($DB1E,$18E1,$F638,$A194,$BFF0));  {-3.8524122716370304260E-5}
const
  ln2m1h: THexExtW = ($0CA8,$5C61,$D010,$9D1B,$BFFD);  {-3.0685281944005469057E-1}
  lnpi2h: THexExtW = ($AE23,$5098,$D92D,$E735,$3FFC);  {0.2257913526447274323630976} {ln(pi/2)/2}
var
  d: array[2..28] of extended absolute dh;
  p: array[2..27] of extended;
  x,sum: extended;
  k: integer;
begin

  if s=1.0 then begin
    etam1pos := extended(ln2m1h); {ln(2)-1}
    exit;
  end
  else if abs(s) <= ETAEPS then begin
    etam1pos := extended(lnpi2h)*s - 0.5;
    exit;
  end;

  x := -s;

  {Calculate p[k] := 1/k^s but only if necessary. Prime powers are}
  {evaluated with power/exp?, otherwise products of p[k] are used.}
  p[2] := exp2(x);
  if s >= 120.0 then begin
    etam1pos := -p[2];
    exit;
  end;

  p[3] := exp3(x);
  p[4] := p[2]*p[2];
  if s >= 50.0 then begin
    etam1pos := (p[3] - p[4]) - p[2];
    exit;
  end;

  p[10] := exp10(x);
  p[ 5] := p[10]/p[2];
  p[ 6] := p[2]*p[3];
  p[ 7] := exp7(x);
  p[ 8] := p[2]*p[4];
  p[ 9] := p[3]*p[3];
  if s >= 28.0 then begin
    sum := 0.0;
    for k:=10 downto 3 do begin
      if odd(k) then sum := sum + p[k]
      else sum := sum - p[k];
    end;
    etam1pos := sum - p[2];
    exit;
  end;

  p[11] := power(11,x);
  p[12] := p[2]*p[6];
  p[13] := power(13,x);
  p[14] := p[2]*p[7];
  p[15] := p[3]*p[5];
  p[16] := p[2]*p[8];
  p[17] := power(17,x);
  p[18] := p[2]*p[9];
  p[19] := power(19,x);
  p[20] := p[2]*p[10];
  p[21] := p[3]*p[7];
  p[22] := p[2]*p[11];
  if s>=19.5 then begin
    sum := 0.0;
    for k:=22 downto 3 do begin
      if odd(k) then sum := sum + p[k]
      else sum := sum - p[k];
    end;
    etam1pos := sum - p[2];
  end
  else begin
    p[23] := power(23,x);
    p[24] := p[3]*p[8];
    p[25] := p[5]*p[5];
    p[26] := p[2]*p[13];
    p[27] := p[3]*p[9];
    {Convergence acceleration: see P. Borwein[36]}

    {The d[] are from P. Borwein's Algorithm 2, but scaled and shifted.}
    {Calculated with Maple VR4 and T_RCalc/xh using n:=27; Digits:=50; }

    { d(n,k) = n*sum((n+j-1)!*4^j/((n-j)!*(2*j)!),j=0..k); }
    { d[2]   := 1/d(n,n)-1.0;                              }
    { d[i]   := (-1)^(i-2)*(d(n,i-2)/d(n,n)-1), i=3..n+1;  }
    sum := p[2]*p[14]*d[28];
    for k:=27 downto 3 do sum := sum + extended(d[k])*p[k];
    etam1pos := sum + d[2]*p[2];
  end;
end;


{---------------------------------------------------------------------------}
function zetap(s,sc: extended): extended;
  {-Return the Riemann zeta function at s>0, s<>1, sc=1-s}
var
  y: extended;
{Based on boost_1_42_0\boost\math\special_functions\zeta.hpp [19]}
{Copyright John Maddock 2007, see 3rdparty.ama for Boost license}
const
  P1: array[0..5] of extended = (
        0.243392944335937499969,
       -0.496837806864865688082,
        0.0680008039723709987107,
       -0.00511620413006619942112,
        0.000455369899250053003335,
       -0.279496685273033761927e-4);
  Q1: array[0..6] of extended = (
        1.0,
       -0.30425480068225790522,
        0.050052748580371598736,
       -0.00519355671064700627862,
        0.000360623385771198350257,
       -0.159600883054550987633e-4,
        0.339770279812410586032e-6);
  P2: array[0..5] of extended = (
        0.577215664901532860605,
        0.222537368917162139445,
        0.0356286324033215682729,
        0.00304465292366350081446,
        0.000178102511649069421904,
        0.700867470265983665042e-5);
  Q2: array[0..6] of extended = (
        1.0,
        0.259385759149531030085,
        0.0373974962106091316854,
        0.00332735159183332820617,
        0.000188690420706998606469,
        0.635994377921861930071e-5,
        0.226583954978371199405e-7);
  P4: array[0..6] of extended = (
       -0.053725830002359501027,
        0.0470551187571475844778,
        0.0101339410415759517471,
        0.00100240326666092854528,
        0.685027119098122814867e-4,
        0.390972820219765942117e-5,
        0.540319769113543934483e-7);
  Q4: array[0..7] of extended = (
        1.0,
        0.286577739726542730421,
        0.0447355811517733225843,
        0.00430125107610252363302,
        0.000284956969089786662045,
        0.116188101609848411329e-4,
        0.278090318191657278204e-6,
       -0.19683620233222028478e-8);
  P7: array[0..7] of extended = (
       -2.49710190602259407065,
       -3.36664913245960625334,
       -1.77180020623777595452,
       -0.464717885249654313933,
       -0.0643694921293579472583,
       -0.00464265386202805715487,
       -0.000165556579779704340166,
       -0.252884970740994069582e-5);
  Q7: array[0..8] of extended = (
        1.0,
        1.01300131390690459085,
        0.387898115758643503827,
        0.0695071490045701135188,
        0.00586908595251442839291,
        0.000217752974064612188616,
        0.397626583349419011731e-5,
       -0.927884739284359700764e-8,
        0.119810501805618894381e-9);
  P15: array[0..8] of extended = (
        -4.78558028495135548083,
        -3.23873322238609358947,
        -0.892338582881021799922,
        -0.131326296217965913809,
        -0.0115651591773783712996,
        -0.000657728968362695775205,
        -0.252051328129449973047e-4,
        -0.626503445372641798925e-6,
        -0.815696314790853893484e-8);
  Q15: array[0..8] of extended = (
         1.0,
         0.525765665400123515036,
         0.10852641753657122787,
         0.0115669945375362045249,
         0.000732896513858274091966,
         0.30683952282420248448e-4,
         0.819649214609633126119e-6,
         0.117957556472335968146e-7,
        -0.193432300973017671137e-12);
  P42: array[0..8] of extended = (
        -10.3948950573308861781,
         -2.82646012777913950108,
         -0.342144362739570333665,
         -0.0249285145498722647472,
         -0.00122493108848097114118,
         -0.423055371192592850196e-4,
         -0.1025215577185967488e-5,
         -0.165096762663509467061e-7,
         -0.145392555873022044329e-9);
  Q42: array[0..9] of extended = (
         1.0,
         0.205135978585281988052,
         0.0192359357875879453602,
         0.00111496452029715514119,
         0.434928449016693986857e-4,
         0.116911068726610725891e-5,
         0.206704342290235237475e-7,
         0.209772836100827647474e-9,
        -0.939798249922234703384e-16,
         0.264584017421245080294e-18);
const
   x1: extended = 81487.0/65536.0;
   x4: extended = 366299.0/524288.0;
begin
  {$ifdef debug}
    if abs((s+sc)-1.0)>5e-19*maxx(1.0,abs(s)) then begin
      if RTE_ArgumentRange>0 then RunError(byte(RTE_ArgumentRange));
      zetap := NaN_x;
      exit;
    end;
  {$endif}
  if s<1.0 then begin
    y := PolEvalX(sc,P1,6)/PolEvalX(sc,Q1,7) - x1;
    zetap := (y+sc)/sc;
  end
  else if s<=2.0 then begin
    s := -sc;
    y := PolEvalX(s,P2,6)/PolEvalX(s,Q2,7);
    zetap := y - 1.0/sc;
  end
  else if s<=4.0 then begin
    s := s-2.0;
    y := PolEvalX(s,P4,7)/PolEvalX(s,Q4,8) + x4;
    zetap := y - 1.0/sc;
  end
  else if s<=7.0 then begin
    s := s-4.0;
    y := PolEvalX(s,P7,8)/PolEvalX(s,Q7,9);
    zetap := 1.0 + exp(y);
  end
  else if s<15.0 then begin
    s := s-7.0;
    y := PolEvalX(s,P15,9)/PolEvalX(s,Q15,9);
    zetap := 1.0 + exp(y);
  end
  else if s<42.0 then begin
    s := s-15.0;
    y := PolEvalX(s,P42,9)/PolEvalX(s,Q42,10);
    zetap := 1.0 + exp(y);
  end
  else if s<64.0 then zetap := 1.0 + exp2(-s)
  else zetap := 1.0;
end;


{---------------------------------------------------------------------------}
function sfc_etaint(n: integer): extended;
  {-Return the Dirichlet function eta(n) for integer arguments}
const
  etahex: array[0..64] of THexExtW = (
            ($0000,$0000,$0000,$8000,$3FFE),  {+5.0000000000000000000E-1}
            ($79AC,$D1CF,$17F7,$B172,$3FFE),  {+6.9314718055994530943E-1} {ln(2)}
            ($9918,$983E,$3312,$D28D,$3FFE),  {+8.2246703342411321821E-1}
            ($8EA3,$4049,$803B,$E6CB,$3FFE),  {+9.0154267736969571407E-1}
            ($F2FE,$EDD3,$BE56,$F270,$3FFE),  {+9.4703282949724591755E-1}
            ($0DC3,$DD50,$D75D,$F8DC,$3FFE),  {+9.7211977044690930596E-1}
            ($26B0,$A8DB,$1389,$FC4D,$3FFE),  {+9.8555109129743510409E-1}
            ($BAB7,$C7BF,$A0EA,$FE1A,$3FFE),  {+9.9259381992283028266E-1}
            ($FBAF,$272C,$2042,$FF09,$3FFE),  {+9.9623300185264789924E-1}
            ($EBAC,$43E9,$1B9E,$FF83,$3FFE),  {+9.9809429754160533077E-1}
            ($BA16,$8BE8,$0D9C,$FFC1,$3FFE),  {+9.9903950759827156566E-1}
            ($0FDC,$1D92,$5B03,$FFE0,$3FFE),  {+9.9951714349806075415E-1}
            ($D69C,$9E16,$1EA1,$FFF0,$3FFE),  {+9.9975768514385819088E-1}
            ($B52F,$23EE,$0A49,$FFF8,$3FFE),  {+9.9987854276326511548E-1}
            ($52EC,$A017,$0372,$FFFC,$3FFE),  {+9.9993917034597971818E-1}
            ($3D85,$74C0,$0127,$FFFE,$3FFE),  {+9.9996955121309923808E-1}
            ($871B,$CD21,$0062,$FFFF,$3FFE),  {+9.9998476421490610646E-1}
            ($BF99,$0379,$8021,$FFFF,$3FFE),  {+9.9999237829204101199E-1}
            ($49E9,$064F,$C00B,$FFFF,$3FFE),  {+9.9999618786961011349E-1}
            ($05F0,$AE11,$E003,$FFFF,$3FFE),  {+9.9999809350817167510E-1}
            ($1CDC,$3A59,$F001,$FFFF,$3FFE),  {+9.9999904661158152213E-1}
            ($52AB,$68DD,$F800,$FFFF,$3FFE),  {+9.9999952325821554283E-1}
            ($B299,$22F9,$FC00,$FFFF,$3FFE),  {+9.9999976161323082254E-1}
            ($E242,$0BA9,$FE00,$FFFF,$3FFE),  {+9.9999988080131843951E-1}
            ($9FF6,$03E3,$FF00,$FFFF,$3FFE),  {+9.9999994039889239462E-1}
            ($F529,$014B,$FF80,$FFFF,$3FFE),  {+9.9999997019885696281E-1}
            ($AC5B,$006E,$FFC0,$FFFF,$3FFE),  {+9.9999998509923199657E-1}
            ($E572,$0024,$FFE0,$FFFF,$3FFE),  {+9.9999999254955048496E-1}
            ($4CD0,$000C,$FFF0,$FFFF,$3FFE),  {+9.9999999627475340009E-1}
            ($19B0,$0004,$FFF8,$FFFF,$3FFE),  {+9.9999999813736941811E-1}
            ($5DEB,$0001,$FFFC,$FFFF,$3FFE),  {+9.9999999906868228147E-1}
            ($74A5,$0000,$FFFE,$FFFF,$3FFE),  {+9.9999999953434033146E-1}
            ($26E2,$0000,$FFFF,$FFFF,$3FFE),  {+9.9999999976716989595E-1}
            ($0CF6,$8000,$FFFF,$FFFF,$3FFE),  {+9.9999999988358485804E-1}
            ($0452,$C000,$FFFF,$FFFF,$3FFE),  {+9.9999999994179239904E-1}
            ($0171,$E000,$FFFF,$FFFF,$3FFE),  {+9.9999999997089618955E-1}
            ($007B,$F000,$FFFF,$FFFF,$3FFE),  {+9.9999999998544809144E-1}
            ($0029,$F800,$FFFF,$FFFF,$3FFE),  {+9.9999999999272404461E-1}
            ($000E,$FC00,$FFFF,$FFFF,$3FFE),  {+9.9999999999636202195E-1}
            ($0005,$FE00,$FFFF,$FFFF,$3FFE),  {+9.9999999999818101087E-1}
            ($0002,$FF00,$FFFF,$FFFF,$3FFE),  {+9.9999999999909050541E-1}
            ($0001,$FF80,$FFFF,$FFFF,$3FFE),  {+9.9999999999954525270E-1}
            ($0000,$FFC0,$FFFF,$FFFF,$3FFE),  {+9.9999999999977262632E-1}
            ($0000,$FFE0,$FFFF,$FFFF,$3FFE),  {+9.9999999999988631316E-1}
            ($0000,$FFF0,$FFFF,$FFFF,$3FFE),  {+9.9999999999994315658E-1}
            ($0000,$FFF8,$FFFF,$FFFF,$3FFE),  {+9.9999999999997157829E-1}
            ($0000,$FFFC,$FFFF,$FFFF,$3FFE),  {+9.9999999999998578914E-1}
            ($0000,$FFFE,$FFFF,$FFFF,$3FFE),  {+9.9999999999999289457E-1}
            ($0000,$FFFF,$FFFF,$FFFF,$3FFE),  {+9.9999999999999644729E-1}
            ($8000,$FFFF,$FFFF,$FFFF,$3FFE),  {+9.9999999999999822364E-1}
            ($C000,$FFFF,$FFFF,$FFFF,$3FFE),  {+9.9999999999999911182E-1}
            ($E000,$FFFF,$FFFF,$FFFF,$3FFE),  {+9.9999999999999955591E-1}
            ($F000,$FFFF,$FFFF,$FFFF,$3FFE),  {+9.9999999999999977796E-1}
            ($F800,$FFFF,$FFFF,$FFFF,$3FFE),  {+9.9999999999999988898E-1}
            ($FC00,$FFFF,$FFFF,$FFFF,$3FFE),  {+9.9999999999999994449E-1}
            ($FE00,$FFFF,$FFFF,$FFFF,$3FFE),  {+9.9999999999999997224E-1}
            ($FF00,$FFFF,$FFFF,$FFFF,$3FFE),  {+9.9999999999999998612E-1}
            ($FF80,$FFFF,$FFFF,$FFFF,$3FFE),  {+9.9999999999999999306E-1}
            ($FFC0,$FFFF,$FFFF,$FFFF,$3FFE),  {+9.9999999999999999653E-1}
            ($FFE0,$FFFF,$FFFF,$FFFF,$3FFE),  {+9.9999999999999999826E-1}
            ($FFF0,$FFFF,$FFFF,$FFFF,$3FFE),  {+9.9999999999999999913E-1}
            ($FFF8,$FFFF,$FFFF,$FFFF,$3FFE),  {+9.9999999999999999957E-1}
            ($FFFC,$FFFF,$FFFF,$FFFF,$3FFE),  {+9.9999999999999999978E-1}
            ($FFFE,$FFFF,$FFFF,$FFFF,$3FFE),  {+9.9999999999999999989E-1}
            ($FFFF,$FFFF,$FFFF,$FFFF,$3FFE)); {+9.9999999999999999994E-1}
var
  m: integer;
begin
  if n>64 then sfc_etaint := 1.0
  else if n>=0 then sfc_etaint := extended(etahex[n])
  else begin
    m := 1-n;
    if odd(m) then sfc_etaint := 0
    else sfc_etaint := sfc_bernoulli(m)/m*exp2m1(m);
  end;
end;


{---------------------------------------------------------------------------}
function sfc_eta(s: extended): extended;
  {-Return the Dirichlet eta function}
var
  z,t: extended;
begin
  if s>=65.0 then sfc_eta := 1.0
  else if (frac(s)=0.0) and (abs(s)<=MaxInt) then sfc_eta := sfc_etaint(round(s))
  else if s >= -ETAEPS then sfc_eta := etam1pos(s)+1.0
  else begin
    z := 1.0-s;
    if s <= -8.0 then begin
      {use eta(s) = (1-2^z)*zeta(s)}
      sfc_eta := sfc_zeta(s)*(1.0-exp2(z));
    end
    else begin
      {Use reflection formula for eta with s < 0}
      t := -exp2m1(z)/exp2m1(-s);
      t := t*cospi(0.5*z);
      t := t*sfc_gamma(z);
      t := t/power(pi,z);
      sfc_eta := t*etam1pos(z)+t;
    end;
  end;
end;


{---------------------------------------------------------------------------}
function sfc_etam1(s: extended): extended;
  {-Return Dirichlet eta(s)-1}
begin
  if s < -ETAEPS then sfc_etam1 := sfc_eta(s)-1.0
  else sfc_etam1 := etam1pos(s);
end;


{---------------------------------------------------------------------------}
function sfc_zeta(s: extended): extended;
  {-Return the Riemann zeta function at s, s<>1}
var
  sc: extended;
{$ifdef VER50}
  a,b: extended;
{$endif}
begin
  {Ref: Boost [19], file zeta.hpp}
  if IsNanOrInf(s) then begin
    if s=PosInf_x then sfc_zeta := 1.0
    else sfc_zeta := NaN_x;
    exit;
  end;
  if frac(s)=0.0 then begin
    if (s > -MaxBernoulli) and  (s <= MaxInt) then begin
      sfc_zeta := sfc_zetaint(round(s));
      exit;
    end;
  end
  else if abs(s) <= 1.6e-10 then begin
    {Small s, especially useful for s<0 to avoid reflection machinery}
    sfc_zeta := (-0.5) - LnSqrt2Pi*s;
    exit;
  end;
  sc := 1.0-s;
  if s<0.0 then begin
    if frac(0.5*s)=0.0 then sfc_zeta := 0.0
    else begin
      {$ifdef VER50}
        a := 2.0*sinPi(0.5*s);
        b := power(TwoPi,-sc);
        a := a*b;
        b := sfc_gamma(sc);
        a := a*b;
        b := zetap(sc,s);
        sfc_zeta := a*b;
      {$else}
        sfc_zeta := 2.0*sinPi(0.5*s)*power(TwoPi,-sc)*sfc_gamma(sc)*zetap(sc,s);
      {$endif}
    end;
  end
  else sfc_zeta := zetap(s,sc);
end;


{---------------------------------------------------------------------------}
function sfc_zeta1p(x: extended): extended;
  {-Return the Riemann zeta function at 1+x, x<>0}
begin
  {Ref: Boost [19], file zeta.hpp}
  if abs(x)<1.0 then sfc_zeta1p := zetap(1.0+x,-x)
  else sfc_zeta1p := sfc_zeta(1.0+x);
end;


{---------------------------------------------------------------------------}
function sfc_zetam1(s: extended): extended;
  {-Return Riemann zeta(s)-1, s<>1}
var
  t: extended;
begin
  if s <= 2.0 then sfc_zetam1 := sfc_zeta(s) - 1.0
  else begin
    if s >= 120.0 then sfc_zetam1 := exp2(-s)
    else begin
      t := 0.5*exp2(s);
      {$ifdef VER50}
        s := etam1pos(s);
        sfc_zetam1 := (1.0 + s*t) / (t - 1.0);
      {$else}
        sfc_zetam1 := (1.0 + etam1pos(s)*t) / (t - 1.0);
      {$endif}
    end;
  end;
end;


{---------------------------------------------------------------------------}
function sfc_zetaint(n: integer): extended;
  {-Return zeta(n) for integer arguments, n<>1}
const
  znhex: array[2..63] of THexExtW = (
           ($9918,$983E,$3312,$D28D,$3FFF),  {+1.6449340668482264364}
           ($09C2,$8031,$0027,$99DD,$3FFF),  {+1.2020569031595942854}
           ($41B6,$3EC2,$9156,$8A89,$3FFF),  {+1.0823232337111381915}
           ($6DBD,$53E6,$0C76,$84BA,$3FFF),  {+1.0369277551433699263}
           ($247C,$0492,$4C26,$8238,$3FFF),  {+1.0173430619844491397}
           ($C46D,$A679,$96D0,$8111,$3FFF),  {+1.0083492773819228268}
           ($B746,$C31C,$9B57,$8085,$3FFF),  {+1.0040773561979443394}
           ($AB81,$C0B5,$CF9E,$8041,$3FFF),  {+1.0020083928260822144}
           ($CBF1,$D2DD,$9719,$8020,$3FFF),  {+1.0009945751278180854}
           ($9954,$F245,$318D,$8010,$3FFF),  {+1.0004941886041194645}
           ($9821,$D966,$1052,$8008,$3FFF),  {+1.0002460865533080483}
           ($DEF6,$E845,$0564,$8004,$3FFF),  {+1.0001227133475784892}
           ($9C2B,$5E56,$01C9,$8002,$3FFF),  {+1.0000612481350587048}
           ($11BF,$BCBF,$0097,$8001,$3FFF),  {+1.0000305882363070205}
           ($9178,$66F5,$8032,$8000,$3FFF),  {+1.0000152822594086518}
           ($A19A,$C1CD,$4010,$8000,$3FFF),  {+1.0000076371976378998}
           ($6E8A,$932A,$2005,$8000,$3FFF),  {+1.0000038172932649999}
           ($F9BA,$DB08,$1001,$8000,$3FFF),  {+1.0000019082127165539}
           ($A233,$9E2C,$0800,$8000,$3FFF),  {+1.0000009539620338727}
           ($ACA0,$34AE,$0400,$8000,$3FFF),  {+1.0000004769329867878}
           ($D9D9,$118C,$0200,$8000,$3FFF),  {+1.0000002384505027277}
           ($F138,$05D8,$0100,$8000,$3FFF),  {+1.0000001192199259653}
           ($CFFF,$01F2,$0080,$8000,$3FFF),  {+1.0000000596081890513}
           ($3A95,$00A6,$0040,$8000,$3FFF),  {+1.0000000298035035146}
           ($662E,$0037,$0020,$8000,$3FFF),  {+1.0000000149015548284}
           ($76B9,$0012,$0010,$8000,$3FFF),  {+1.0000000074507117898}
           ($2768,$0006,$0008,$8000,$3FFF),  {+1.0000000037253340248}
           ($0D18,$0002,$0004,$8000,$3FFF),  {+1.0000000018626597235}
           ($AF05,$0000,$0002,$8000,$3FFF),  {+1.0000000009313274324}
           ($3A56,$0000,$0001,$8000,$3FFF),  {+1.0000000004656629064}
           ($1372,$8000,$0000,$8000,$3FFF),  {+1.0000000002328311834}
           ($067B,$4000,$0000,$8000,$3FFF),  {+1.0000000001164155017}
           ($0229,$2000,$0000,$8000,$3FFF),  {+1.0000000000582077209}
           ($00B8,$1000,$0000,$8000,$3FFF),  {+1.0000000000291038504}
           ($003D,$0800,$0000,$8000,$3FFF),  {+1.0000000000145519218}
           ($0014,$0400,$0000,$8000,$3FFF),  {+1.0000000000072759598}
           ($0007,$0200,$0000,$8000,$3FFF),  {+1.0000000000036379796}
           ($0002,$0100,$0000,$8000,$3FFF),  {+1.0000000000018189896}
           ($0001,$0080,$0000,$8000,$3FFF),  {+1.0000000000009094948}
           ($0000,$0040,$0000,$8000,$3FFF),  {+1.0000000000004547474}
           ($0000,$0020,$0000,$8000,$3FFF),  {+1.0000000000002273737}
           ($0000,$0010,$0000,$8000,$3FFF),  {+1.0000000000001136868}
           ($0000,$0008,$0000,$8000,$3FFF),  {+1.0000000000000568434}
           ($0000,$0004,$0000,$8000,$3FFF),  {+1.0000000000000284217}
           ($0000,$0002,$0000,$8000,$3FFF),  {+1.0000000000000142108}
           ($0000,$0001,$0000,$8000,$3FFF),  {+1.0000000000000071054}
           ($8000,$0000,$0000,$8000,$3FFF),  {+1.0000000000000035527}
           ($4000,$0000,$0000,$8000,$3FFF),  {+1.0000000000000017764}
           ($2000,$0000,$0000,$8000,$3FFF),  {+1.0000000000000008882}
           ($1000,$0000,$0000,$8000,$3FFF),  {+1.0000000000000004441}
           ($0800,$0000,$0000,$8000,$3FFF),  {+1.0000000000000002220}
           ($0400,$0000,$0000,$8000,$3FFF),  {+1.0000000000000001110}
           ($0200,$0000,$0000,$8000,$3FFF),  {+1.0000000000000000555}
           ($0100,$0000,$0000,$8000,$3FFF),  {+1.0000000000000000278}
           ($0080,$0000,$0000,$8000,$3FFF),  {+1.0000000000000000139}
           ($0040,$0000,$0000,$8000,$3FFF),  {+1.0000000000000000069}
           ($0020,$0000,$0000,$8000,$3FFF),  {+1.0000000000000000035}
           ($0010,$0000,$0000,$8000,$3FFF),  {+1.0000000000000000017}
           ($0008,$0000,$0000,$8000,$3FFF),  {+1.0000000000000000009}
           ($0004,$0000,$0000,$8000,$3FFF),  {+1.0000000000000000004}
           ($0002,$0000,$0000,$8000,$3FFF),  {+1.0000000000000000002}
           ($0001,$0000,$0000,$8000,$3FFF)); {+1.0000000000000000001}
begin
  if n>63 then sfc_zetaint := 1.0
  else if n>1 then sfc_zetaint := extended(znhex[n])
  else if n<0 then sfc_zetaint := sfc_bernoulli(1-n)/(n-1)+0.0  {avoid -0}
  else if n=0 then sfc_zetaint := -0.5
  else sfc_zetaint := PosInf_x;
end;


{---------------------------------------------------------------------------}
function hz_a1(s,a: extended; var ok: boolean): extended;
  {-Return zetah(s,a) for a close to 1, s<>1 (normally s < 0)}
var
  x,t,f,z: extended;
  n: integer;
const
  MaxIter = 100;
begin
  hz_a1 := 0.0;
  ok := false;
  {compute sum from NIST[30], 25.11.10}
  z := sfc_zeta(s);
  if IsNanOrInf(z) then exit;
  x := 1.0-a;
  if x<>0.0 then begin
    n := 0;
    f := 1;
    repeat
      f := f*(s+n);
      inc(n);
      t := s+n;
      if (n>MaxIter) or (t=1.0) then exit;
      f := f*x/n;
      t := sfc_zeta(t);
      t := t*f;
      z := z + t;
      {do not use t for exit test because zeta(s+n) may be very small}
    until (abs(f)<=eps_x*abs(z)) and (n>2);
  end;
  ok := true;
  hz_a1 := z;
end;


{---------------------------------------------------------------------------}
function hurwitz_formula(s,a: extended): extended;
  {-Compute zetah with range reduction and Hurwitz formula, a > 0, s < 0}
var
  n,k: longint;
  a2,f,h,r,t,tol,sh,z: extended;
  done: boolean;
const
  MaxIter = 10000;
begin
  r := 0.0;
  z := -s;
  f := frac(a);
  k := trunc(a);
  if f=0.0 then begin
    {Use strict Hurwitz formula with reduced a in (0,1].}
    {Empirically it is valid also for reduced frac(a)=0.}
    f := 1.0;
    k := k-1;
  end;
  a2  := 2.0*f;
  sh  := 2.0*frac(0.25*s);  {sh = s/2 mod 2}
  tol := 0.5*eps_x;
  {NIST [30],25.11.4}
  for n:=0 to k-1 do begin
    t := power(f+n,z);
    r := r + t;
  end;
  h := 0.0;
  {Perform summation only for non-trivial arguments}
  if (frac(a2)<>0.0) or (frac(sh)<>0.0) then begin
    {A. Erdelyi [50], Higher Transcendental Functions I, 1.10 (6),}
    {see also NIST [30], 25.11.9  with s <-> 1-s and cos <-> sin. }
    n := 1;
    z := s-1.0;
    done := false;
    repeat
      t := n*a2 + sh;
      if frac(t)<>0.0 then begin
        t := sinpi(t);
        f := power(n, z);
        t := t*f;
        h := h + t;
        done := (f < tol*(abs(h) + 1e-10));  {1e-10 addition to avoid exit if h ~ 0}
      end;
      inc(n);
    until done or (n>MaxIter);
    if (n>MaxIter) and (RTE_NoConvergence>0) then RunError(byte(RTE_NoConvergence));
  end;
  if h=0.0 then hurwitz_formula := -r
  else begin
    {compute = 2*h*Gamma(1-s)/(2Pi)^(1-s)}
    z := 1.0 - s;
    if (z>0.0) and (z<MAXGAMX) then begin
      f := sfc_gamma(z);
      t := power(TwoPi, -z);
      t := (t*f)*(2.0*h);
    end
    else begin
      f := sfc_lngamma(z);
      t := ln(abs(2.0*h)) + f - z*ln(TwoPi);
      if t>= ln_MaxExt then t := PosInf_x
      else t := exp(t);
      if h<0.0 then t := -t;
    end;
    hurwitz_formula := t - r;
  end
end;


{---------------------------------------------------------------------------}
function bernpoly_intern(n: integer; x: extended): extended;
  {-Return the Bernoulli polynomial B_n(x), internal use: for small n only}
var
  i: integer;
  s,f: extended;
  ref: boolean;
begin
  if n<0 then bernpoly_intern := 0
  else if n=0 then bernpoly_intern := 1.0
  else if n=1 then bernpoly_intern := x - 0.5
  else begin
    ref := false;
    if abs(x-1.0) <= 0.125 then begin
      {if x is near 1 use NIST[30], 24.4.3: B_n(1-x) = (-1)^n B_n(x)}
      x := 1.0 - x;
      ref := odd(n);
    end;
    {Compute sum NIST[30], 24.2.5}
    f := n;
    s := x - 0.5*n;
    for i:=2 to n do begin
      f := (n+1-i)*f/i;
      s := s*x;
      if i and 1 = 0 then s := s + sfc_bernoulli(i)*f;
    end;
    if ref then s := -s;
    bernpoly_intern := s;
  end;
end;


{---------------------------------------------------------------------------}
function sfc_zetah(s,a: extended): extended;
  {-Return the Hurwitz zeta function zetah(s,a) = sum(1/(i+a)^s, i=0..INF), s<>1, a>0}
var
  eps,q,r,t,u,w,z: extended;
  j,k,n: integer;
  ok: boolean;
const
  MaxIter = 40;
  eps1 = 1e-4;   {Threshold for hz_a1}
  S_HF = -8;     {Threshold for Hurzwitz formula}
label
  noconv;
begin
  {Initial reference: Cephes [7], file double\zeta.c}
  if IsNanOrInf(s) or IsNanOrInf(a) or (s=1.0) or (a<=0.0) then begin
    {$ifopt R+}
      if RTE_ArgumentRange>0 then RunError(byte(RTE_ArgumentRange));
    {$endif}
    sfc_zetah := NaN_x;
    exit;
  end
  else if s=0.0 then begin
    sfc_zetah := 0.5 - a;       {NIST [30], 25.11.13}
    exit;
  end
  else if a=0.5 then begin
    if s < 16384.0 then begin
      sfc_zetah := exp2m1(s)*sfc_zeta(s);   {NIST [30], 25.11.11}
      exit;
    end;
  end
  else if a=1.0 then begin
    sfc_zetah := sfc_zeta(s);   {NIST [30], 25.11.2}
    exit;
  end
  else if a=2.0 then begin
    sfc_zetah := sfc_zetam1(s); {NIST [30], 25.11.2/3}
    exit;
  end;

  if s>0.0 then begin
    {Quick checks: avoid overflow/underflow}
    t := ln(a);
    t := -s*t;
    if t>ln_MaxExt then begin
      sfc_zetah := PosInf_x;
      exit;
    end
    else if t<ln_MinExt then begin
      {here s > 1, 1/a^s ~ 0, avoid underflow in some cases:}
      {zetah ~ a^(1-s)/(s-1) + 0.5/a^s = (a/(s-1) + 0.5)/a^s}
      {      = a^(1-s)/(s-1) + 0.5*a^(1-s)/a}
      u := power(a, 1-s);
      sfc_zetah := u/(s-1.0) + 0.5*u/a;
      exit;
    end;
  end
  else begin
    if (a>1.0) then begin
      if -s*ln(a-1.0) >ln_MaxExt then begin
        sfc_zetah := PosInf_x;
        exit;
      end;
    end;
    {Test if Bernoulli polynomials or Hurwitz formula can be used}
    if (s >= S_HF) and (frac(s)=0.0) then begin
      {s small negative integer: use 'naive' Bernoulli polynomial code}
      k := 1 - round(s);
      sfc_zetah := -bernpoly_intern(k,a)/k;  {NIST [30], 25.11.14}
      exit;
    end;
    if abs(a-1.0)<eps1 then begin
      {try expansion near a=1}
      sfc_zetah := hz_a1(s,a,ok);
      if ok then exit;
    end;
    if s < S_HF then begin
      sfc_zetah := hurwitz_formula(s,a);
      exit;
    end
  end;

  {Calculate z = zetah with the Euler-Maclaurin summation formula:}
  {See e.g. http://functions.wolfram.com/10.02.06.0020.01         }
  {z = sum1(s,a,n) + (a+n)^(1-s)/(s-1) - 1/2/(a+n)^s + sum2(s,a,n)}
  {with sum1(a,n) = sum(1/(a+k)^s, k=0..n) and                    }
  {sum2 = sum(B_2k/(2k)! * s(s+1)..(s+2k)/(a+n)^(s+2k+1),k=0..INF)}

  eps := 0.5*eps_x;
  if abs(s)<0.5 then n := 8
  else n := 9;

  {$ifdef Delphi}
    {Avoid brain-damaged paranoid warning: r,w,t might not have been initialized}
    r := 0; w := 0; t := 0;
  {$endif}

  {compute z = s1(a,n) = sum(1/(a+k)^s, k=0..n)}
  z := 0.0;
  for k:=0 to n do begin
    w := a+k;
    r := power(w,-s);
    z := z+r;
    if r < eps*z then begin
      sfc_zetah := z;
      exit;
    end
  end;

  {Add the two single terms z = z + (a+n)^(1-s)/(s-1) - 1/2/(a+n)^s}
  {Here w=(a+n), r=1/(a+n)^s}
  z := z + r*w/(s-1.0);
  z := z - 0.5*r;

  {Add the terms of sum2 until a term is < e*(partial result)}
  q := 1.0;
  k := 0;
  for j:=0 to MaxIter do begin
    q := q*(s+k);
    r := r/w;
    if j>NBoF then begin
      {Some problematic (s,a) values: Check decreasing terms in}
      {asymptotic expansion and increase convergence tolerance.}
      if j=NBOF+1 then eps := 1.5*eps_x;
      u := t;
      t := sfc_bernoulli(k+2);  {TP5 fix}
      t := t/sfc_fac(k+2);
      t := (q*t)*r;
      if abs(u)<=abs(t) then goto noconv;
    end
    else t := (q*extended(BoFHex[j]))*r;
    z := z + t;
    if abs(t) < eps*abs(z) then begin
      sfc_zetah := z;
      exit;
    end;
    q := q*(s+k+1);
    r := r/w;
    inc(k,2);
  end;

noconv:
  {No convergence}
  sfc_zetah := z;
  {$ifdef debug}
    writeln('sfc_zetah - Euler-Maclaurin issue: ',s:21,' ',a:21);
  {$endif}
  if RTE_NoConvergence>0 then RunError(byte(RTE_NoConvergence));
end;


{---------------------------------------------------------------------------}
function bernpoly_rev(n: integer; x: extended): extended;
  {-Return the Bernoulli polynomial B_n(x), internal use: n>1, small x}
var
  i: integer;
  s,t,u,f: extended;
begin
  {This code runs NIST[30] 24.2.5 backwards}
  s := sfc_bernoulli(n);
  f := 1.0;
  t := PosInf_x;
  for i:=n-1 downto 0 do begin
    f := f*x*(i+1)/(n-i);
    if (i and 1 = 0) or (i=1) then begin
      {B_i <> 0}
      u := t;
      t := s;
      s := t + sfc_bernoulli(i)*f;
      if (s=t) and (s=u) then begin
        {return if 3 partial sums have the same FP value}
        bernpoly_rev := s;
        exit;
      end;
    end;
  end;
  bernpoly_rev := s;
end;


{---------------------------------------------------------------------------}
function sfc_bernpoly(n: integer; x: extended): extended;
  {-Return the Bernoulli polynomial B_n(x), 0 <= n <= MaxBernoulli}
var
  a: extended;
  neg: boolean;
begin
  if IsNan(x) or (n<0) or (n>MaxBernoulli) then begin
    {$ifopt R+}
      if RTE_ArgumentRange>0 then RunError(byte(RTE_ArgumentRange));
    {$endif}
    sfc_bernpoly := Nan_x;
    exit;
  end;
  if n=0 then sfc_bernpoly := 1.0
  else if n=1 then sfc_bernpoly := x-0.5
  else if (x=0.0) or (x=1.0) then sfc_bernpoly := sfc_bernoulli(n)  {n<>1 !}
  else if x=0.5 then begin
    {B_n(1/2) = (2^(1-n)-1) B_n)}
    if n>65 then a := -1
    else a := exp2m1(1-n);
    sfc_bernpoly := a*sfc_bernoulli(n)
  end
  else if (x=-1.0) then begin
    {B_n(-1) := B_n + (-1)^n*n}
    if odd(n) then sfc_bernpoly := -n
    else sfc_bernpoly := sfc_bernoulli(n) + n;
  end
  else begin
    a := 1-x;
    if abs(a) > 0.5 then neg := false
    else begin
      {B_n(1-x) = (-1)^n B_n(x)}
      x := a;
      neg := odd(n);
    end;
    a := abs(x);
    if a > 12000.0*n then begin
      {Large x: First terms from NIST[30], 24.4.12 with x=0.5 and h = x-0.5}
      {B_n(x) = (x-0.5)^n - n(n-1)/24*(x-0.5)^(n-2) + 7/240*binomial(n,4)*(x-0.5)^(n-4) + ...}
      a := power(x-0.5,n-2);
      a := a*(sqr(x-0.5) - n*(n-1)/24.0);
    end
    else begin
      if n<=10 then begin
        {small degree}
        a := bernpoly_intern(n,x);
      end
      else if a < 0.125/n then begin
        {small argument}
        a := bernpoly_rev(n,x);
      end
      else begin
        {Loop count in Hurwitz formula is ~ trunc(abs(x)),}
        if x>=0.0 then a := sfc_zetah(1-n, x)
        else begin
          a := sfc_zetah(1-n, 1.0-x);
          if odd(n) then neg := not neg;
        end;
        a := -a*n;
      end;
    end;
    if neg then a := -a;
    sfc_bernpoly := a;
  end;
end;


{---------------------------------------------------------------------------}
function lphi_aj(z, s, a: extended): extended;
  {-Lerch Phi via Aksenov/Jentschura convergence acceleration}
const
  IMAX  = 50;  {Max. iteration count for CNCT case }
  IMAX2 = 90;  {Max. iteration for direct summation}
var
  num,den,SAj : array[0..IMAX] of extended;

  {---------------------------------------------------------------------------}
  function vwaj(j: integer): extended;
     {-Compute the van Wijngaarden quantities A_j from b^j_k}
  var
    sum, bjk, z2ind: extended;
    ind, two2k: double;
  begin
    sum  := 0.0;
    two2k:= 1.0;
    {Sum b^j_k's over k}
    repeat
      {Index for the term of the original series}
      ind   := two2k*(j+1)-1.0;
      z2ind := power(z,ind);
      bjk   := two2k*z2ind/power(a+ind,s);
      sum   := sum + bjk;
      two2k := 2.0*two2k;
    until abs(bjk) <= eps_x*abs(sum);
    vwaj := sum;
  end;

var
  j,i,k,sign: integer;
  sn,eps0,eps,skn,skn0,omega,fk,est,t,x: extended;
{$ifndef V7Plus}
label
  continue;
{$endif}
begin

  {Based on the C code lerchphi.c by S.V. Aksenov and U.D. Jentschura.}
  {The original specs are heavily modified: s and a must be positive, }
  {eps=0 test is moved to avoid division by zero, num[0]=den[0]=0 is  }
  {allowed on the 1st iteration etc. The code for |z|<=0.5 is not used}
  {by the driver function, but is left here for testing and comparing.}

  {The CNCT is described in [40], C source and lerchphi user guide is }
  {available from http://aksenov.freeshell.org/lerchphi.html          }

  lphi_aj := 0.0;
  x := abs(z);

  {$ifdef debug}
    if (z >= 1.0) or (z < -1.0) or (a<=0.0) or (s<=0.0)  then begin
      {Return NaN_ this should not happen if called by sfc_lerch}
      lphi_aj := NaN_x;
      exit;
    end;
  {$endif}

  if x <= MinExtended then begin
    {return first term of series}
    lphi_aj := power(a, -s);
    exit;
  end;

  {sn denotes current partial sum of defining series:
    z >  0.5: sn is partial sum S_n of the van Wijngaarden transformed series.
    z <= 0.5: sn is the partial sum of the power series defining LerchPhi.
  skn0 and skn denote successive partial sums S^k_n that are same as sn
  in case of direct summation and delta-transformed in case of CNCT.
  eps0 and eps denote successive differences between partial sums S^k_n.}

  eps0 := 0.0;
  skn  := 0.0;
  skn0 := 0.0;
  sn   := 0.0;

  {omega is next term of a partial sum (of defining power series for direct  }
  {summation, of van Wijngaarden transformed series for CNCT and also becomes}
  {a remainder estimate in the delta transformation in CNCT.}

  {For z<=0.5 van Wijngaarden transformation is not used hence no calls to aj()}
  if z <= 0.5 then omega := power(a, -s)
  else omega := vwaj(0);

  if IsInf(omega) then begin
    lphi_aj := omega;
    exit;
  end;
  i    := -1;
  sign := -1;

  {Main loop: iterations for S^k_n}
  repeat
    {$ifndef V7Plus}
      continue:  {label}
    {$endif}
    {i points to current iterate}
    inc(i);
    sign := -sign;
    sn   := sn + omega;

    {Next term: omega}
    if z <= 0.5 then omega := z*power((a+i)/(a+i+1), s)*omega
    else begin
      {CNCT (z > 0.5) case}
      SAj[i] := sign*omega;
      if odd(i) then omega := vwaj(i+1)
      else begin
        j := i div 2;
        omega := 0.5*(SAj[j] - power(z,j)/power(a+j,s));
      end;
      omega := -sign*omega;
    end;

    if x <= 0.5 then begin
      {Direct summation case: store current}
      skn := sn;
    end
    else begin
      {CNCT case: Make sure omega is representable machine number}
      if abs(omega) <= MinExtended then begin
        num[i] := 0;
        den[i] := 0;
        if i=0 then begin
          {WE: Allow zeros for i=0}
          {$ifndef V7Plus}goto{$endif} continue;
        end
        else begin
          if RTE_NoConvergence>0 then RunError(byte(RTE_NoConvergence));
          exit;
        end;
      end
      else begin
        t := 1.0/omega;
        {Last terms in sums of numerator and denominator of i-th partial sum}
        num[i] := sn*t;
        den[i] := t;
      end;
      {Recurrence computation of numerator and denominator of a S_k^n}
      k  := i;
      fk := (k+1)*k;
      for j:=i-1 downto 0 do begin
        t := fk/(k*(k+1));
        num[j] := num[j+1] - t*num[j];
        den[j] := den[j+1] - t*den[j];
        inc(k);
      end;
      {Current approximation of the sum S_k^n}
      skn := num[0]/den[0];
    end; {CNCT case}
    lphi_aj := skn;
    eps := abs(skn - skn0);
    {Check the three termination criteria}

    {Successive iterates skn are the same. WE: must be   }
    {checked first, otherwise division by zero may occur!}
    if eps=0.0 then exit;

    {|est/skn| is less than the requested accuracy
    (est is a remainder estimate)}
    if (i>0) and (eps < eps0) then begin
      if x > 0.5 then begin
        est := eps/eps0;
        est := 2.0/est/(1.0-est)*eps;
      end
      else est := 2.0*power(x, i+1)/power(a+i+1, s);
      if abs(est) < 8.0*eps_x*abs(skn) then exit;
    end;

    {Maximum number of iterations is exceeded}
    if i>=imax then begin
      if (x>0.5) or (i>=IMAX2) then begin
        if RTE_NoConvergence>0 then RunError(byte(RTE_NoConvergence));
        exit;
      end;
    end;

    {Go on to the next iteration}
    skn0 := skn;
    eps0 := eps;

  until false;

end;


{---------------------------------------------------------------------------}
function lphi_aea(z,s,a: extended): extended;
  {-Lerch Phi, asymptotic expansion for large a>0}
var
  n: integer;
  e,r,t,x,y: extended;
const
  NMAX=1000;
begin
  {This function uses the asymptotic expansion given in [41], Theorem 1.}
  {Note: The simple convergence test is not accurate for |z| close to 1,}
  {in this case a test using the error term in [41] should be used.     }
  n := 1;
  r := 1.0/(1.0-z);
  x := power(a,-s);
  if x<>0 then begin
    y := 1.0;
    e := 0.5*eps_x;
    repeat
      {y holds the terms (s)_n/(n!*a^n)}
      y := y*(s/n)/a;
      t := sfc_polylog(-n,z);
      t := y*t;
      if odd(n) then r := r - t
      else r := r + t;
      inc(n);
      s := s+1.0;
    until (n>NMAX) or (abs(t) < e*abs(r));
  end;
  lphi_aea := x*r;
  if (n>NMAX) and (RTE_NoConvergence>0) then RunError(byte(RTE_NoConvergence));
end;


{---------------------------------------------------------------------------}
function lphi_sum(z,s,a: extended): extended;
  {-Lerch Phi, direct summation sum(z^n/(n+a)^s, n=0...)}
var
  n: integer;
  r,t,x,e: extended;
const
  NMAX=1000;
begin
  n := 0;
  x := 1.0; {x = z^n}
  r := 0.0;
  e := 0.5*eps_x;
  s := -s;
  repeat
    t := x*power(a+n,s);
    r := r + t;
    n := n+1;
    x := x*z;
  until (n>NMAX) or (abs(t) <= e*abs(r));
  lphi_sum := r;
  if (n>NMAX) and (RTE_NoConvergence>0) then RunError(byte(RTE_NoConvergence));
end;


{---------------------------------------------------------------------------}
function sfc_lerch(z,s,a: extended): extended;
  {-Return the Lerch transcendent Phi(z,s,a) = sum(z^n/(n+a)^s, n=0..INF), |z|<=1, s>=0, a>0; s>1 if z=1}
var
  x,t: extended;
begin
  {Phi(z,s,a) = sum(z^n/(n+a)^s, n=0...)}
  if IsNanOrInf(z) or IsNanOrInf(s) or IsNanOrInf(a)
     or (abs(z)>1.0) or (s<0.0) or (a<=0.0) then
  begin
    {$ifopt R+}
      if RTE_ArgumentRange>0 then RunError(byte(RTE_ArgumentRange));
    {$endif}
    sfc_lerch := NaN_x;
    exit;
  end;

  if s=0.0 then begin
    {geometric series Phi = sum(z^n)}
    sfc_lerch := 1.0/(1.0-z);
    exit;
  end;

  x := abs(z);
  if x=1.0 then begin
    if z<0 then begin
      {Although Aksenov/Jentschura do not allow z=-1, this is a classical}
      {series transformation scenario and their code gives good results. }
      {The alternative Phi = (zetah(s,a/2)-zetah(s,(a+1)/2))/2^s is very }
      {susceptible to truncation errors and gives 0 for a>2/eps_x!       }
      sfc_lerch := lphi_aj(z,s,a);
    end
    else begin
      if s<=1.0 then sfc_lerch := PosInf_x
      else sfc_lerch := sfc_zetah(s,a);
    end;
    exit;
  end;

  t := s*ln(a);
  if t > ln_MaxExt then begin
    {1/a^s = 0 accurate to extended precision}
    sfc_lerch := 0;
  end
  else if (x <= 0.3) and (a >= 100.0) and (s <= 100.0) then begin
    {Use asymptotic expansion for large a, here only if s and x are not too large}
    sfc_lerch := lphi_aea(z,s,a);
  end
  else if (s >= 30.0) or (t < -43.7) then begin
    if (x>0.5) and (s < 0.1*a) then begin
      {accelerated summation because direct summation needs many iterations}
      sfc_lerch := lphi_aj(z,s,a);
    end
    else begin
      {direct summation if s is large enough or 1/a^s > 1/eps_x}
      sfc_lerch := lphi_sum(z,s,a);
    end;
  end
  else if (1.0-x)*a >= 50.0  then begin
    {Remaining case for asymptotic expansion for large a}
    sfc_lerch := lphi_aea(z,s,a);
    exit;
  end
  else begin
    if (x<=0.5) or (s<=-30) then begin
      {direct summation, lphi_aj would do the same but a lot more complicated}
      sfc_lerch := lphi_sum(z,s,a);
    end
    else begin
      {accelerated summation}
      sfc_lerch := lphi_aj(z,s,a);
    end;
  end;
end;


{---------------------------------------------------------------------------}
function sfc_dbeta(s: extended): extended;
  {-Return the Dirichlet beta function sum((-1)^n/(2n+1)^s, n=0..INF)}
var
  t: extended;
const
  d0: THexExtW = ($B90E,$F218,$0F6B,$C87F,$3FFD); {0.39159439270683677647194534689911102809021011577}
begin
  {http://en.wikipedia.org/wiki/Dirichlet_beta_function   }
  {http://mathworld.wolfram.com/DirichletBetaFunction.html}
  {This is also known as the Catalan beta function        }
  
  if abs(s) <= 1e-10 then sfc_dbeta := 0.5 + extended(d0)*s
  else if s<0.0 then begin
    if frac(0.5*s)=-0.5 then sfc_dbeta := 0.0
    else begin
      t := cosPi(0.5*s);
      s := 1.0 - s;
      t := sfc_dbeta(s)*t;
      sfc_dbeta := sfc_gamma(s)*t*power(Pi_2,-s);
    end;
  end
  else if s>=40.5 then sfc_dbeta := 1.0
  else if s>=27.6 then sfc_dbeta := 1.0 - exp3(-s)
  else if s>=22.8 then sfc_dbeta := 1.0 - exp3(-s) + exp5(-s)
  else begin
    sfc_dbeta := sfc_lerch(-1.0,s,0.5)*exp2(-s);
  end;
end;


{---------------------------------------------------------------------------}
function sfc_dlambda(s: extended): extended;
  {-Return the Dirichlet lambda function sum(1/(2n+1)^s, n=0..INF), s<>1}
begin
  {http://mathworld.wolfram.com/DirichletLambdaFunction.html}
  sfc_dlambda := -exp2m1(-s)*sfc_zeta(s);
end;


{---------------------------------------------------------------------------}
function sfc_lchi(s, x: extended): extended;
  {-Return Legendre's Chi-function chi(s,x); s>=0, |x|<=1, x<>1 if s<=1}
var
  z: extended;
begin
  if IsNanOrInf(s) or IsNanOrInf(x) or (s<0.0) or (abs(x) > 1.0) then begin
    {$ifopt R+}
      if RTE_ArgumentRange>0 then RunError(byte(RTE_ArgumentRange));
    {$endif}
    sfc_lchi := NaN_x;
    exit;
  end;
  {chi(s,x) = sum(x^(2n+1)/(2n+1)^s, n=0..INF)}
  {chi(s,x) = 2^(-s)*x*Phi(x^2,s,1/2) = 0.5*(Li_s(x)-Li_s(-x))}
  {see e.g. http://en.wikipedia.org/wiki/Legendre_chi_function}
  {or  http://mathworld.wolfram.com/LegendresChi-Function.html}
  if x=0.0 then sfc_lchi := 0.0
  else if s>=22.8 then begin
    if s>=40.5 then sfc_lchi := x
    else begin
      z := sqr(x);
      if s>=27.6 then sfc_lchi := x*(1.0 + z*exp3(-s))
      else sfc_lchi := x*(1.0 + z*(exp3(-s) + z*exp5(-s)));
    end;
  end
  else begin
    if s=0.0 then sfc_lchi := x/((1.0+x)*(1.0-x))
    else if s=1.0 then sfc_lchi := arctanh(x)
    else sfc_lchi := sfc_lerch(x*x,s,0.5)*x*exp2(-s);
  end;
end;



end.

